Package: disbayes
Title: Bayesian Multi-State Modelling of Chronic Disease Burden Data
Date: 2026-04-24
Version: 1.1.1
Authors@R: 
    person("Christopher", "Jackson", , "chris.jackson@mrc-bsu.cam.ac.uk", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0002-6656-8913"))
Description: Estimation of incidence and case fatality for a chronic disease, given partial information, using a multi-state model. Given data on age-specific mortality and either incidence or prevalence, Bayesian inference is used to estimate the posterior distributions of incidence, case fatality, and functions of these such as prevalence.  The methods are described in Jackson et al. (2023) <doi:10.1093/jrsssa/qnac015>.
License: GPL-3
Encoding: UTF-8
LazyData: true
Biarch: true
Depends: R (>= 3.5.0)
Imports: dplyr, tidyr, magrittr, tibble, generics, methods, Rcpp (>=
        0.12.0), rstan (>= 2.26.0), mgcv, SHELF, ggplot2, loo,
        matrixStats
LinkingTo: BH (>= 1.66.0), Rcpp (>= 0.12.0), RcppParallel, RcppEigen
        (>= 0.3.3.3.0), rstan (>= 2.26.0), StanHeaders (>= 2.26.0)
Suggests: knitr, rmarkdown, rstantools (>= 2.0.0.9000), tempdisagg,
        testthat, codetools
VignetteBuilder: knitr
SystemRequirements: GNU make
URL: https://chjackson.github.io/disbayes/
BugReports: https://github.com/chjackson/disbayes/issues
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-04-24 15:02:37 UTC; Chris J
Author: Christopher Jackson [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-6656-8913>)
Maintainer: Christopher Jackson <chris.jackson@mrc-bsu.cam.ac.uk>
Repository: CRAN
Date/Publication: 2026-04-27 08:30:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-01 19:43:48 UTC; unix
Archs: disbayes.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(loo,disbayes)
S3method(plot,disbayes)
S3method(plot,disbayes_hier)
S3method(print,disbayes)
S3method(summary,disbayes)
S3method(tidy,disbayes)
S3method(tidy,disbayes_hier)
export(ci2num)
export(conflict_disbayes)
export(disbayes)
export(disbayes_hier)
export(loo)
export(loo_indiv)
export(looi_disbayes)
export(plotfit_data_disbayes)
export(plotfit_disbayes)
export(tidy)
export(tidy_obsdat)
import(Rcpp)
import(dplyr)
import(methods)
import(tibble)
importFrom(generics,tidy)
importFrom(loo,loo)
importFrom(magrittr,"%>%")
importFrom(matrixStats,colQuantiles)
importFrom(mgcv,jagam)
importFrom(mgcv,s)
importFrom(mgcv,smoothCon)
importFrom(rstan,sampling)
importFrom(stats,coef)
importFrom(stats,dbinom)
importFrom(stats,lm)
importFrom(stats,na.omit)
importFrom(stats,pgamma)
importFrom(stats,qbeta)
importFrom(stats,qexp)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rbeta)
importFrom(stats,rlnorm)
importFrom(stats,rnorm)
importFrom(stats,setNames)
importFrom(stats,uniroot)
importFrom(tidyr,extract)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
useDynLib(disbayes, .registration = TRUE)

disbayes
======

The development repository for the `disbayes` R package for chronic disease epidemiology estimation with incomplete data. 

* `disbayes` can estimate age-specific case fatality for a disease, given:

  - published information on age-specific mortality and at least one of incidence or prevalence

  - some indication of the uncertainty associated with the published estimates, either as a credible interval, or by expressing the estimate as a number of cases with associated denominator. 
  
* The underlying model is a three-state multi-state model with states given by no disease, disease and death.  Remission from the disease is optional.

* Case fatality, incidence or remission rates can be modelled as smooth functions of age, through a spline model, or estimated independently for each age.  Case fatality or remission can also be modelled as age-constant.

* Two alternative estimation methods can be used, both based on the [Stan](https://mc-stan.org) software.

	- exact point estimation using optimisation to obtain the posterior mode, with credible intervals based on an approximation to the Bayesian posterior.  This is generally instant to compute, but the uncertainty quantification is approximate. 

	- full Bayesian estimation using Markov Chain Monte Carlo.  This gives more accurate uncertainty quantification but is computationally intensive. 

* The following more advanced models are provided, which are all more computationally intensive:

  - hierarchical models for data by age and area, which share information between areas to strengthen estimates from areas with less data
  
  - hierarchical models for data by age, area and gender, where the effect of gender is assumed to be the same for every area
  
  - models with assumed trends in disease incidence or case fatality through calendar time, where trends can be age-specific (non-hierarchical models only)

* It is inspired by the [DisMod II](https://www.epigear.com/index_files/dismod_ii.html) and [DisMod-MR](https://github.com/ihmeuw/dismod_mr) packages used for the Global Burden of Disease studies.   It modifies and extends the formal, fully Bayesian framework described in the [book by Flaxman et al.](https://uwapress.uw.edu/book/9780295991849/an-integrative-metaregression-framework-for-descriptive-epidemiology/). 

* The method is fully described in [Jackson et al. (2023)](https://doi.org/10.1093/jrsssa/qnac015).

* Source code is at the [GitHub repository](https://github.com/chjackson/disbayes)

## Installation

### CRAN version

```r
install.packages("disbayes")
```


### Development version 

```r
install.packages("devtools") # if devtools not already installed
library(devtools)
install_github("chjackson/disbayes")
```

If this fails, make sure that the `rstan` package is set up properly, as [explained here](https://github.com/stan-dev/rstan/wiki/RStan-Getting-Started).  If you are on Windows, then follow these instructions for [installing rstan from source on Windows](https://github.com/stan-dev/rstan/wiki/Configuring-C---Toolchain-for-Windows).


## Introduction and worked example

[Bayesian estimation of chronic disease epidemiology from incomplete data: the disbayes package](https://chjackson.github.io/disbayes/articles/disbayes.html)



<!-- badges: start -->
[![R-CMD-check](https://github.com/chjackson/disbayes/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/chjackson/disbayes/actions/workflows/R-CMD-check.yaml)
[![test-coverage](https://github.com/chjackson/disbayes/actions/workflows/test-coverage.yaml/badge.svg)](https://app.codecov.io/gh/chjackson/disbayes)
[![CRAN status](https://www.r-pkg.org/badges/version/disbayes)](https://CRAN.R-project.org/package=disbayes)
<!-- badges: end -->

#' The 'disbayes' package.
#'
#' @description Bayesian evidence synthesis for chronic disease epidemiology
#'
#' @name disbayes-package
#' @useDynLib disbayes, .registration = TRUE
#' @import methods
#' @import Rcpp
#' @importFrom stats coef lm qbeta qexp qnorm quantile rlnorm rnorm rbeta dbinom na.omit
#' @importFrom rstan sampling
#' @importFrom mgcv smoothCon jagam s
#' @importFrom matrixStats colQuantiles
#' @import dplyr
#' @importFrom tidyr extract pivot_wider pivot_longer
#' @importFrom magrittr "%>%"
#' @import tibble
#'
#' @references
#' Stan Development Team (2019). RStan: the R interface to Stan. R package version 2.19.2. https://mc-stan.org
#'
"_PACKAGE"

#' @importFrom generics tidy
#' @export
generics::tidy

#' @importFrom loo loo
#' @export
loo::loo

##' Ischemic heart disease in England 
##'
##' @format A data frame with columns:
##'
##' \code{sex}: \code{"male"} or \code{"female"}. 
##'
##' \code{ageyr}. Year of age. 
##'
##' \code{location}. Name of the location, which is either a city region or region in England. 
##'
##' \code{num_mort}. Numerator behind the estimate of mortality 
##' 
##' \code{num_inc}. Numerator behind the estimate of incidence
##' 
##' \code{num_prev}. Numerator behind the estimate of prevalence
##' 
##' \code{denom_mort}. Denominator behind the estimate of mortality 
##' 
##' \code{denom_inc}.  Denominator behind the estimate of incidence
##' 
##' \code{denom_prev}. Denominator behind the estimate of prevalence
##'
##' @source Global Burden of Disease, 2017
##'
##' @details The data were processed to
##'
##' * change the geography to refer to England city regions and the remaining English regions,
##'
##' * change counts by 5-year age groups to estimated 1-year counts,
##'
##' * obtain estimated numerators and denominators from the published point estimates and uncertainty intervals.
##' A point estimate of the risk is equivalent to the numerator divided by the denominator. The denominator is
##' related to the extent of uncertainty around this estimate, and obtained using the Bayesian method
##' implemented in \code{\link{ci2num}}. 
##'
##' The script given in \url{https://github.com/chjackson/disbayes/blob/master/data-raw/gbd_process.Rmd} shows
##' these steps.
##'
##' @references Jackson C, Zapata-Diomedi B, Woodcock J. "Bayesian multistate modelling of incomplete chronic disease burden data" \url{https://arxiv.org/abs/2111.14100}.
##' 
##' @keywords datasets
"ihdengland"



##' Trends in ischemic heart disease in England 
##'
##' @format A data frame with columns:
##'
##' \code{gender}: \code{"male"} or \code{"female"}. 
##'
##' \code{age}: Year of age. 
##'
##' \code{year}: Calendar year.
##'
##' \code{p2017}: Estimated ratio between the outcome in the calendar
##' year and the outcome in 2017.
##'
##' \code{outcome}: Outcome referred to (incidence or case fatality). 
##'
##' @source Scarborough, P., Wickramasinghe, K., Bhatnagar, P. and Rayner, M. (2011) Trends in coronary heart disease, 1961-2001. British Heart Foundation.
##'
##' Smolina, K., Wright, F. L., Rayner, M. and Goldacre, M. J. (2012) Determinants of the decline in mortality from acute myocardial infarction in England between 2002 and 2010: linked national database study. BMJ, 344.
##'
##' British Heart Foundation (2020) Heart and Circulatory Disease Statistics 2020. British Heart Foundation.
##'
##' @details The data were interpolated and smoothed to produce a matrix by year of age and
##' calendar year, using the script at \url{https://github.com/chjackson/disbayes/blob/master/data-raw/trends.r}.
##'
##' @keywords datasets
"ihdtrends"

##' Convert a proportion and credible interval to a numerator and denominator
##'
##' Estimate the number of events and denominator that contain roughly equivalent information to an estimate and uncertainty interval for a proportion, by interpreting the estimate and interval as a Beta posterior arising from a vague Beta(0.5,0.5) prior updated with the data consisting of that number and denominator. 
##'
##' Based on fitting a Beta distribution by least squares, using the method provided by the \pkg{SHELF} package.
##'
##' Requires that the estimate and upper and lower limits are all distinct (except that \code{est=0} is allowed and handled specially for convenience, see \code{denom0}).  Vectors of estimates and limits may be supplied. 
##'
##' @param est Point estimate
##'
##' @param lower Lower 95\% credible limit 
##'
##' @param upper Upper 95\% credible limit 
##'
##' @param epsilon If any of \code{lower} are zero, then they are replaced by the minimum of \code{epsilon} and \code{est/2}.  Similarly values of 1 for \code{upper} are replaced by the maximum of \code{1-epsilon} and \code{(1+est)/2}.
##'
##' @param denom0 Denominator to use as a default when the point estimate is exactly 0 or 1 (which is not compatible with the beta distribution).  Should correspond to a guess of the population size used to produce the estimate,
##' which should be no greater than the actual population of the area, and usually less.  Should be either a scalar, or a vector of the same length as \code{est} (though note if it is a vector, then only the elements where \code{est} is 1 or 0 get used). 
##'
##'
##' @return A data frame with elements \code{num} and \code{denom} corresponding to the supplied estimate and limits. 
##' 
##' @examples 
##' est <- 3.00 / 100 
##' upper <- 3.52 / 100 
##' lower <- 2.60 / 100 
##' ci2num(est, lower, upper)
##'
##' @references Oakley (2020). SHELF: Tools to Support the Sheffield Elicitation Framework. R package version 1.7.0. \url{https://CRAN.R-project.org/package=SHELF}
##'
##' @export
ci2num <- function(est, lower, upper, epsilon=0.5, denom0=1000){
    check_proportion_ci2num(est, "est")
    check_proportion_ci2num(lower, "lower")
    check_proportion_ci2num(upper, "upper")
    check_interval_ci2num(est, lower, upper)
    r <- n <- numeric(length(est))
    zeroinds = which(est==0)
    r[zeroinds] <- 0
    n[zeroinds] <- rep(denom0, length.out=length(est))[zeroinds]

    inds <- which(est > 0)
    if (length(inds) > 0) { 
        est <- est[inds]
        lower <- lower[inds]
        upper <- upper[inds]
        lower[lower==0] <- pmin(epsilon[lower==0], est[lower==0]/2)
        upper[upper==1] <- pmax(1-epsilon[upper==1], (1+est[upper==1])/2)
        vals <- rbind(lower, est, upper)
        probs <- c(0.025, 0.5, 0.975)
        bet <- SHELF::fitdist(vals=vals, probs=probs, lower=0, upper=1)$Beta
        apost <- bet$shape1
        bpost <- bet$shape2
        aprior <- bprior <- 0.5
        r[inds] <- round(apost - aprior)
        n[inds] <- round(bpost - bprior + r[inds])
    }
    data.frame(num=r, denom=n)
}

check_proportion_ci2num <- function(x, prefix){
    badprop <- which(x < 0 | x > 1)
    if (length(badprop) > 0) {
        stop(sprintf("%s[%s]=%s should be in [0,1]", 
                     prefix, badprop[1], x[badprop[1]]))
    }
}

check_interval_ci2num <- function(x, lower, upper){
    badint <- which(lower > upper)
    if (length(badint) > 0) {
        stop(sprintf("lower[%s]=%s, should be < upper[%s]=%s",
                     badint[1], lower[badint[1]], badint[1], upper[badint[1]]))
    }    
    badint <- which((x!=0) & (x <= lower | x >= upper))
    if (length(badint) > 0) {
        stop(sprintf("est[%s]=%s should be inside the credible interval of (lower[%s]=%s, upper[%s]=%s)",
                     badint[1], x[badint[1]], 
                     badint[1], lower[badint[1]], 
                     badint[1], upper[badint[1]]))
    }
}

test_that("ci2num", {
    
    est <- c(0.2, 0.3, 0.4)
    lower <- c(0.1, 0.2, 0.3)
    upper <- c(0.3, 0.4, 0.5)
    expect_equal(ci2num(est, lower, upper)$num, c(13,24,36))
    
    ciz1 <- ci2num(0, 0, 1, denom0=100)
    expect_equal(ciz1$num, 0)
    expect_equal(ciz1$denom, 100)
    
    expect_equal(ci2num(0.1, 0, 1)$num, 9)
    
    expect_error(ci2num(-1, 0.1, 0.2), "should be in \\[0,1\\]")
    expect_error(ci2num(0.1, 2, 0.2), "should be in \\[0,1\\]")
    expect_error(ci2num(0.1, 0.01, 2), "should be in \\[0,1\\]")
    expect_error(ci2num(0.01, 0.1, 0.2), "should be inside the credible interval")
    expect_error(ci2num(0.01, 0.2, 0.1), "should be < upper")
})    

ihdbristol <- ihdengland %>% filter(location=="Bristol", sex=="Male")

## Smooth incidence.  Noticeably slower than model with just smooth CF

dbres <- disbayes(dat = ihdbristol,
                    inc_num = "inc_num", inc_denom = "inc_denom",
                    prev_num = "prev_num", prev_denom = "prev_denom",
                    mort_num = "mort_num", mort_denom = "mort_denom",
                  eqage = 40,
                  smooth_inc = TRUE, 
                  chains = 1, iter_train = 400, iter=1000, loo=FALSE)


## Prevalence not supplied

dbres <- disbayes(dat = ihdbristol, 
                  inc_num = "inc_num", inc_denom = "inc_denom",
                  mort_num = "mort_num", mort_denom = "mort_denom",
                  eqage = 40, smooth_inc = FALSE, 
                  chains = 1, iter_train = 400, iter=1000, loo=FALSE)

## Prevalence supplied, but not incidence. smooth incidence
dbres2 <- disbayes(dat = ihdbristol, 
                  prev_num = "prev_num", prev_denom = "prev_denom",
                  mort_num = "mort_num", mort_denom = "mort_denom",
                  eqage = 40, smooth_inc = TRUE, 
                  chains = 1, iter_train = 400, iter=1000, loo=FALSE)

# Unsmoothed model doesn't actually do too bad  
dbres <- disbayes(dat = ihdbristol, 
                  prev_num = "prev_num", prev_denom = "prev_denom",
                  mort_num = "mort_num", mort_denom = "mort_denom",
                  eqage = 40, smooth_inc = FALSE, 
                  chains = 1, iter_train = 400, iter=1000, loo=FALSE)

summ <- tidy(dbres)
summ <- tidy(dbres2)
summ %>% filter(var=="inc") %>%
  ggplot(aes(x=age,y=`50%`,col=model,group=model)) + 
  geom_line() + geom_point() +
  ylim(0,0.1) + 
  geom_ribbon(aes(ymin=`2.5%`,ymax=`97.5%`), alpha=0.5)

summ %>% filter(var=="cf") %>%
  ggplot(aes(x=age,y=`50%`,col=model,group=model)) + 
  geom_line() + geom_point() +
  geom_ribbon(aes(ymin=`2.5%`,ymax=`97.5%`), alpha=0.5)

## Empirical Bayes method
## Call disbayes to estimate posterior modes of hyperparameters
## Return the modes so that they can subsequently be fixed to fit the desired model 

eb_find_modes <- function(dbcall, dbfn=disbayes, hp_fixed_arg){ 
    dbcall$method <- "opt"
    dbcall$hessian <- FALSE
    dbcall$draws <- 0
    dbcall$hp_fixed <- hp_fixed_arg
    dbcall[[1]] <- NULL
    ## Don't allow extra args to be passed through (e.g. to customise the sampler)
    fargs <- names(dbcall)[names(dbcall) %in% names(formals(dbfn))]
    dbcall <- dbcall[fargs]
    ebres <- do.call(dbfn, dbcall)
    ebres$fit$par
}

expand_hpfixed <- function(hp, hp_fixed){
  hpfnew <- vector(nrow(hp), mode="list")
  names(hpfnew) <- hp$pars
  for (i in seq_along(hp$pars)) {
    hpfnew[[i]] <- if (is.null(hp_fixed[[hp$pars[i]]])) FALSE else hp_fixed[[hp$pars[i]]]
  }
  hpfnew
}

.disbayes_hier_hp <- data.frame(
  pars      = c("scf","sinc","srem","scfmale","sd_int","sd_slope"),
  row.names = c("scf","sinc","srem","scfmale","sd_int","sd_slope"),
  stannames = c("lambda_cf[1]", "lambda_inc[1]", "lambda_rem[1]", 
                "lambda_cf_male[1]", "sd_inter[1]", "sd_slope[1]"),
  stringsAsFactors=FALSE
)

.disbayes_hp <- data.frame(
  pars      = c("scf","sinc","srem"),
  row.names = c("scf","sinc","srem"),
  stannames = c("lambda_cf[1]","lambda_inc[1]","lambda_rem[1]"),
  stringsAsFactors=FALSE
)

eb_disbayes <- function(hplist, hp_fixed, dbcall, dbfn, method, dotargs){
  hp <- hplist
  hp$include <- TRUE
  hp_fixed <- expand_hpfixed(hp, hp_fixed)
  hp$vals <- 1
  for (i in seq_along(hp_fixed)) if (is.numeric(hp_fixed[[i]])) hp$vals[i] <- hp_fixed[[i]]
  normalapprox_wanted <- ((method=="opt") && isTRUE(dotargs$hessian) && 
                            !is.null(dotargs$draws) && dotargs$draws> 1)
  unc_wanted <- (method %in% c("mcmc","vb") || normalapprox_wanted)
  hp$eb <- sapply(hp_fixed, isTRUE) # parameters to do empirical Bayes on
  if (unc_wanted && any(hp$eb)) {
    hp_fixed_arg <- hp_fixed
    hp_fixed_arg[hp$eb] <- FALSE
    modes <- eb_find_modes(as.list(dbcall), dbfn=dbfn, hp_fixed_arg)
    hp$vals[hp$eb] <- modes[hp$stannames[hp$eb]]
  } else modes <- NULL 
  tmp <- sapply(hp_fixed, function(x){is.numeric(x) || isTRUE(x)})
  hp$isfixed <- tmp
  hp
}

##' Bayesian estimation of chronic disease epidemiology from incomplete data
##'
##' Estimates a three-state disease model from incomplete data. 
##' It is designed to estimate case fatality and incidence, given
##' data on mortality and at least one of incidence and prevalence.
##' Remission may also be included in the data and modelled.
##'
##'
##' @rdname disbayes
##' @md
##'
##' @param data  Data frame containing some of the variables below.  The
##'   variables below are provided as character strings naming columns in this
##'   data frame.   For each disease measure available, one of the following three
##'   combinations of variables must be specified:
##'
##'   (1) numerator and denominator (2) estimate and denominator (3) estimate
##'   with lower and upper credible limits.
##'   
##'   Mortality must be supplied, and at least one of incidence and prevalence. 
##'   If remission is assumed to be possible, then remission data should also be supplied (see below).
##'
##'   Estimates refer to the probability of having some event within a year, rather than rates.  Rates per year $r$ can be
##'   converted to probabilities $p$ as $p = 1 - exp(-r)$, assuming the rate is constant
##'   within the year. 
##'
##'   For estimates based on registry data assumed to cover the whole
##'   population, then the denominator will be the population size.
##'
##' @param inc_num Numerator for the incidence data, assumed to represent the
##'   observed number of new cases within a year among a population of size
##'   \code{inc_denom}.
##'
##' @param inc_denom Denominator for the incidence data.
##'
##'   The function \code{\link{ci2num}} can be used to convert a published
##'   estimate and interval for a proportion to an implicit numerator and
##'   denominator.
##'
##'   Note that to include extra uncertainty beyond that implied by a published
##'   interval, the numerator and denominator could be multiplied by a constant,
##'   for example, multiplying both the numerator and denominator by 0.5 would
##'   give the data source half its original weight.
##'
##' @param inc_prob Estimate of the incidence probability
##' @param inc_lower Lower credible limit for the incidence estimate
##' @param inc_upper Upper credible limit for the incidence estimate
##'
##' @param prev_num Numerator for the estimate of prevalence, i.e.
##' number of people currently with a disease. 
##' 
##' @param prev_denom Denominator for the estimate of prevalence (e.g. the size
##'   of the survey used to obtain the prevalence estimate)
##' @param prev_prob Estimate of the prevalence probability
##' @param prev_lower Lower credible limit for the prevalence estimate
##' @param prev_upper Upper credible limit for the prevalence estimate
##'
##' @param mort_num Numerator for the estimate of the mortality probability, i.e
##' number of deaths attributed to the disease under study within a year 
##' 
##' @param mort_denom Denominator for the estimate of the mortality probability (e.g.
##'   the population size, if the estimates were obtained from a comprehensive
##'   register)
##' 
##' @param mort_prob  Estimate of the mortality probability
##' @param mort_lower Lower credible limit for the mortality estimate
##' @param mort_upper Upper credible limit for the mortality estimate
##'
##' @param rem_num Numerator for the estimate of the remission probability, i.e number
##' of people observed to recover from the disease within a year. 
##'
##' Remission
##'   data should be supplied if remission is permitted in the model, either as
##'   a numerator and denominator or as an estimate and lower credible interval.
##'   Conversely, if no remission data are supplied, then remission is assumed
##'   to be impossible.  These "data" may represent a prior judgement rather than
##'   observation - lower denominators or wider credible limits represent
##'   weaker prior information. 
##'   
##' @param rem_denom Denominator for the estimate of the remission probability
##' @param rem_prob  Estimate of the remission probability
##' @param rem_lower Lower credible limit for the remission estimate
##' @param rem_upper Upper credible limit for the remission estimate
##'
##' @param age Variable in the data indicating the year of age.  This must
##' start at age zero, but can end at any age. 
##'
##' @param cf_model Model for how case fatality rate varies with age.
##'
##' \code{"smooth"} (the default). Case fatality rate is modelled as a smooth function of age,
##'   using a spline.
##'
##' \code{"indep"} Case fatality rates are estimated independently for each year of age.  This may be
##' useful for determining how much information is in
##'   the data. That is, if the posterior from this model is identical to the
##'   prior for a certain age, then there is no information in the data alone
##'   about case fatality at that age, indicating that some other structural
##'   assumption (such as a smooth function of age) or external data are equired
##'   to give more precise estimates.
##'
##' \code{"increasing"} Case fatality rate is modelled as a smooth and increasing
##'   function of age.
##'
##' \code{"const"} Case fatality rate is modelled as constant with age.
##'
##' @param inc_model Model for how incidence rates vary with age.
##'
##' \code{"smooth"} (the default). Incidence rates are modelled as a smooth spline function of age.
##'
##' \code{"indep"} Incidence rates for each year of age are estimated independently.
##'
##' @param rem_model Model for how remission rates vary with age, which are typically
##' less well-informed by data, compared to incidence and case fatality. 
##'
##' \code{"const"} (the default). Constant remission rate over all ages.
##'
##' \code{"smooth"} Remission rates are modelled as a smooth spline function of age. 
##'
##' \code{"indep"} Remission rates estimated independently over all ages. 
##'
##' @param prev_zero If \code{TRUE}, attempt to estimate prevalence at age zero
##'   from the data, as part of the Bayesian model, even if the observed prevalence is zero.
##'   Otherwise (the default) this is assumed to be zero if the count is zero, and estimated
##'   otherwise. 
##'
##' @param inc_trend Matrix of constants representing trends in incidence
##'   through calendar time by year of age.  There are \code{nage} rows and
##'   \code{nage} columns, where \code{nage} is the number of years of age
##'   represented in the data. The entry in the ith row and jth column
##'   represents the ratio between the incidence \code{nage+j} years prior to
##'   the year of the data, year, and the incidence in the year of the data, for
##'   a person i-1 years of age. For example, if \code{nage=100} and the data
##'   refer to the year 2017, then the first column refers to the year 1918 and
##'   the last (100th) column refers to 2017.  The last column should be all 1,
##'   unless the current data are supposed to be biased.
##'
##'   To produce this format from a long data frame, filter to the appropriate
##'   outcome and subgroup, and use \code{\link[tidyr]{pivot_wider}}, e.g.
##'
##'   \code{trends <- ihdtrends %>% 
##'            filter(outcome=="Incidence", gender=="Female") %>%
##'             pivot_wider(names_from="year", values_from="p2017") %>% 
##'             select(-age, -gender, -outcome) %>% 
##'             as.matrix()}
##'
##' @param cf_trend Matrix of constants representing trends in case fatality
##'   through calendar time by year of age, in the same format as
##'   \code{inc_trend}.
##'
##' @param cf_init Initial guess at a typical case fatality value, for an
##'   average age.
##'
##' @param eqage Case fatalities (and incidence and remission rates) are assumed to be equal for
##'   all ages below this age, inclusive, when using the smoothed model.
##'
##' @param eqagehi Case fatalities (and incidence and remission rates) are assumed to be equal for
##'   all ages above this age, inclusive, when using the smoothed model.
##'
##' @param sprior Rates of the exponential prior distributions used to penalise
##'   the coefficients of the spline model.   The default of 1 should adapt
##'   appropriately to the data, but Higher values give stronger smoothing, or
##'   lower values give weaker smoothing,  if required.
##'
##'   This can be a named vector with names \code{"inc","cf","rem"} in any
##'   order, giving the prior smoothness rates for incidence, case fatality and
##'   remission.  If any of these are not smoothed they can be excluded, e.g.
##'   \code{sprior = c(cf=10, inc=1)}.
##'
##'   This can also be an unnamed vector of three elements, where the first
##'   refers to the spline model for incidence, the second for case fatality,
##'   the third for remission. If one of the rates (e.g. remission) is not being
##'   modelled with a spline, any number can be supplied here and it is just
##'   ignored.
##'
##' @param hp_fixed A list with one named element for each hyperparameter
##' to be fixed.  The value should be either 
##' 
##' * a number (to fix the hyperparameter at this number) 
##' 
##' * \code{TRUE} (to fix the hyperparameter at the posterior mode from a training run
##' where it is not fixed)
##' 
##' If the element is either \code{NULL}, \code{FALSE}, or omitted from the list, 
##' then the hyperparameter is given a prior and estimated as part of the Bayesian model.  
##' 
##' The hyperparameters that can be fixed are 
##' 
##' * \code{scf} Smoothness parameter for the spline relating case fatality to age.
##' 
##' * \code{sinc} Smoothness parameter for the spline relating incidence to age.  
##'
##' For example, to fix the case fatality smoothness to 1.2 and fix the incidence
##' smoothness to its posterior mode, 
##' specify \code{hp_fixed = list(scf=1.2, sinc=TRUE)}.
##' 
##' @param inc_prior Vector of two elements giving the Gamma shape and rate parameters of the
##' prior for the incidence rate.  Only used if \code{inc_model="indep"}, for each age-specific rate. 
##' 
##' @param cf_prior Vector of two elements giving the Gamma shape and rate parameters of the
##' prior for the case fatality rate.  Only used if \code{cf_model="const"}, or if \code{cf_model="indep"}, for each age-specific rate,
##' and for the rate at \code{eqage} in \code{cf_model="increasing"}.
##'
##' @param rem_prior Vector of two elements giving the Gamma shape and rate parameters of the
##' prior for the remission rate, used in both \code{rem_model="const"} and \code{rem_model="indep"}. 
##'
##' @param method String indicating the inference method, defaulting to
##'   \code{"opt"}.
##'
##'   If \code{method="mcmc"} then a sample from the posterior is drawn using Markov Chain Monte Carlo
##'   sampling, via \pkg{rstan}'s \code{\link[rstan:stanmodel-method-sampling]{rstan::sampling()}} function.   This is the most
##'   accurate but the slowest method.
##'
##'   If \code{method="opt"}, then instead of an MCMC sample from the posterior,
##'   `disbayes` returns the posterior mode calculated using optimisation, via
##'   \pkg{rstan}'s \code{\link[rstan:stanmodel-method-optimizing]{rstan::optimizing()}} function.
##'   A sample from a normal approximation to the (real-line-transformed)
##'   posterior distribution is drawn in order to obtain credible intervals.
##' 
##'   If the optimisation fails to converge (non-zero return code), try increasing the
##'   number of iterations from the default 1000, e.g. `disbayes(...,
##'   iter=10000, ...)`, or changing the algorithm to `disbayes(...,
##'   algorithm="Newton", ...)`.
##'
##'   If there is an error message that mentions `chol`, then
##'   the computed Hessian matrix is not positive definite at the reported optimum, hence credible intervals
##'   cannot be computed.
##'   This can occur either because of numerical error in computation of the Hessian, or because the
##'   reported optimum is wrong.  If you are willing to believe
##'   the optimum and live without credible intervals, then set \code{draws=0} to skip
##'   computation of the Hessian.   To examine the problematic Hessian, set
##'   \code{hessian=TRUE,draws=0}, then look at the \code{$fit$hessian} component of the
##'   `disbayes` return object.   If it can be inverted, do \code{sqrt(diag(solve()))} on the Hessian, and
##' check for \code{NaN}s, indicating the problematic parameters. 
##' Otherwise, diagonal entries of the Hessian matrix that are very small
##'   may indicate parameters that are poorly identified from the data, leading to computational
##'   problems. 
##' 
##'   If \code{method="vb"}, then variational Bayes methods are used, via \pkg{rstan}'s
##'   \code{\link[rstan:stanmodel-method-vb]{rstan::vb()}} function.  This is labelled as "experimental" by
##'   \pkg{rstan}.  It might give a better approximation to the posterior
##'   than \code{method="opt"}, but has not been investigated much for `disbayes` models.
##'
##' @param draws Number of draws from the normal approximation to the posterior
##' when using \code{method="opt"}.  
##'
##' @param iter Number of iterations for MCMC sampling, or maximum number of iterations for optimization.
##'
##' @param stan_control (\code{method="mcmc"} only). List of options supplied as the \code{control} argument
##'   to \code{\link[rstan:stanmodel-method-sampling]{rstan::sampling()}} in \pkg{rstan} for the main model fit.
##'   
##'
##' @param bias_model Experimental model for bias in the incidence estimates due
##'   to conflicting information between the different data sources.  If
##'   \code{bias_model=NULL} (the default) no bias is assumed, and all data are
##'   assumed to be generated from the same age-specific incidences.
##'
##'   Otherwise there are assumed to be two alternative curves of incidence by
##'   age (denoted 2 and 1) where curve 2 is related to curve 1 via a constant
##'   hazard ratio that is estimated from the data, given a standard normal
##'   prior on the log scale.  Three distinct curves would not be identifiable
##'   from the data. 
##'
##'   If \code{bias_model="inc"} then the incidence data is assumed to be
##'   generated from curve 2, and the prevalence and mortality data from curve
##'   1.
##'
##'   \code{bias_model="prev"} then the prevalence data is generated from curve
##'   2, and the incidence and mortality data from curve 1.
##'
##'   If \code{bias_model="incprev"} then both incidence and prevalence data are
##'   generated from curve 2, and the mortality data from curve 1.
##'
##' @param ... Further arguments passed to \code{\link[rstan:stanmodel-method-sampling]{rstan::sampling()}} to
##'   control MCMC sampling, or \code{\link[rstan:stanmodel-method-optimizing]{rstan::optimizing()}} to control
##'   optimisation, in Stan.
##'
##' @return A list including the following components
##'
##'  \code{call}: Function call that was used. 
##'
##'  \code{fit}: An object containing posterior samples from the fitted model,
##'   in the \code{stanfit} format returned by the \code{\link[rstan]{stan}}
##'   function in the \pkg{rstan} package.
##'
##'  \code{method}:  Optimisation method that was chosen.
##'
##'  \code{nage}: Number of years of age in the data
##'  
##'  \code{dat}: A list containing the input data in the form of numerators
##'   and denominators.
##' 
##'  \code{stan_data}: Full list of data supplied to Stan
##' 
##'  \code{stan_inits}: Full list of parameter initial values supplied to Stan
##'
##'  \code{hp_fixed} Values of any hyperparameters that are fixed during the main model fit. 
##'
##'   Use the \code{\link{tidy.disbayes}} method to return summary statistics
##'   from the fitted models, simply by calling \code{tidy()} on the fitted model. 
##'   
##' @references Jackson C, Zapata-Diomedi B, Woodcock J. (2023)
##' "Bayesian multistate modelling of incomplete chronic disease burden data"
##' Journal of the Royal Statistical Society, Series A, 186(1), 1-19
##' \doi{10.1093/jrsssa/qnac015}
##'
##' @export
disbayes <- function(data,
                     inc_num=NULL, inc_denom=NULL, inc_prob=NULL, inc_lower=NULL, inc_upper=NULL,
                     prev_num=NULL, prev_denom=NULL, prev_prob=NULL, prev_lower=NULL, prev_upper=NULL,
                     mort_num=NULL, mort_denom=NULL, mort_prob=NULL, mort_lower=NULL, mort_upper=NULL,
                     rem_num=NULL, rem_denom=NULL, rem_prob=NULL, rem_lower=NULL, rem_upper=NULL,
                     age="age",
                     cf_model="smooth",
                     inc_model="smooth",
                     rem_model="const",
                     prev_zero=FALSE, 
                     inc_trend = NULL, 
                     cf_trend = NULL, 
                     cf_init = 0.01,
                     eqage = 30,
                     eqagehi = NULL,
                     sprior = c(1,1,1),
                     hp_fixed = NULL, 
                     rem_prior = c(1.1, 1), 
                     inc_prior = c(2, 0.1), 
                     cf_prior = c(2, 0.1), 
                     method = "opt",
                     draws = 1000,
                     iter = 10000,
                     stan_control = NULL, 
                     bias_model = NULL,
                     ...
){
  dbcall <- match.call()
  check_age(data, age)
  nage <- nrow(data)
  check_rate_prior(inc_prior, missing(inc_prior), inc_model %in% c("indep"), "\"indep\"", "inc")
  check_rate_prior(cf_prior, missing(cf_prior), (cf_model %in% c("const", "indep","increasing")),
                   "\"const\", \"indep\" or \"increasing\"", "cf")
  check_rate_prior(rem_prior, missing(rem_prior), rem_model %in% c("const","indep"), "\"const\" or \"indep\"", "rem")
  
  ## Convert all data to numerators and denominators 
  inc_data <- process_data(data, "inc", inc_num, inc_denom, inc_prob, inc_lower, inc_upper, nage)
  prev_data <- process_data(data, "prev", prev_num, prev_denom, prev_prob, prev_lower, prev_upper, nage)
  if (!inc_data$inc_supplied && !prev_data$prev_supplied)
    stop("At least one of incidence or prevalence should be supplied")
  mort_data <- process_data(data, "mort", mort_num, mort_denom, mort_prob, mort_lower, mort_upper, nage)
  if (!mort_data$mort_supplied)
    stop("Mortality data should be supplied")
  rem_data <- process_data(data, "rem", rem_num, rem_denom, rem_prob, rem_lower, rem_upper, nage)
  remission <- rem_data$rem_supplied
  dat <- c(inc_data, prev_data, mort_data, rem_data, nage=nage, remission=as.numeric(remission))
  
  cf_model <- match.arg(cf_model, c("smooth", "indep", "increasing", "const"))
  inc_model <- match.arg(inc_model, c("smooth", "indep"))
  rem_model <- match.arg(rem_model, c("const", "smooth", "indep"))
  smooth_cf <- (cf_model=="smooth")
  increasing_cf <- (cf_model=="increasing")
  const_cf <- (cf_model=="const")
  const_rem <- (rem_model=="const")
  smooth_inc <- (inc_model=="smooth")
  smooth_rem <- (remission && rem_model=="smooth")
  
  if (!is.null(inc_trend) || !is.null(cf_trend)) {
    trend <- TRUE
    smooth_cf <- TRUE
  } else trend <- FALSE
  
  if (trend) {
    if (is.null(inc_trend)) inc_trend <- matrix(1, nage, nage)
    if (is.null(cf_trend)) cf_trend <- matrix(1, nage, nage)
    check_trenddata(inc_trend, nage)
    check_trenddata(cf_trend, nage)
  }
  prev_zero <- prev_zero || (!is.null(dat$prev_num) && dat$prev_num[1] > 0) 
  
  check_eqage(eqage, eqagehi, nage)
  mdata <- list(remission=remission, eqage=eqage, const_rem=const_rem,
                smooth_rem=smooth_rem, prev_zero=prev_zero,
                inc_prior=inc_prior, cf_prior=cf_prior, rem_prior=rem_prior)
  idata <- list(cf_init=cf_init) 
  sprior <- check_sprior(sprior)
  
  initrates <- init_rates(dat, mdata, idata, ...)
  
  if (increasing_cf) smooth_cf <- TRUE
  if (const_cf) increasing_cf <- TRUE

  # Handle fixed hyperparameters
  check_hp_fixed(hp_fixed, .disbayes_hp, inc_model, cf_model, rem_model, ng=1)
  if (inc_model %in% c("indep") && !is.null(hp_fixed[["sinc"]])) hp_fixed[["sinc"]] <- NULL
  if (cf_model %in% c("indep","const") && !is.null(hp_fixed[["scf"]])) hp_fixed[["scf"]] <- NULL
  if (rem_model %in% c("indep","const") && !is.null(hp_fixed[["srem"]])) hp_fixed[["srem"]] <- NULL
  hp <- eb_disbayes(.disbayes_hp, hp_fixed, dbcall, disbayes, method, list(...))

  if (!is.null(bias_model))
      bias_model <- match.arg(bias_model, c("inc","prev","incprev"))
  nbias <- if (is.null(bias_model)) 1 else 2
  if (nbias==1){
      incdata_ind <- prevdata_ind <- 1 
  } else if (nbias==2){
      incdata_ind <-  if (bias_model %in% c("inc", "incprev")) 2 else 1
      prevdata_ind  <-  if (bias_model %in% c("prev", "incprev")) 2 else 1
  }
  
  if (smooth_cf | smooth_inc | smooth_rem) {
    cf_smooth <- if (smooth_cf) init_smooth(log(initrates$cf), eqage, eqagehi, s_opts=NULL) else NULL
    inc_smooth <- if (smooth_inc) init_smooth(log(initrates$inc), eqage, eqagehi, s_opts=NULL) else NULL
    rem_smooth <- if (smooth_rem) init_smooth(log(initrates$rem), eqage, eqagehi, s_opts=NULL) else NULL
    X <- cf_smooth$X
    if (is.null(X)) X <- if (is.null(inc_smooth$X)) rem_smooth$X else inc_smooth$X


    datstans <- c(dat, list(
                           smooth_cf=as.numeric(smooth_cf),
                            increasing_cf=as.numeric(increasing_cf),
                            const_cf=as.numeric(const_cf),
                            const_rem=as.numeric(const_rem),
                            smooth_inc=as.numeric(smooth_inc),
                            smooth_rem=as.numeric(smooth_rem),
                            prev_zero=as.numeric(prev_zero),
                            trend=as.numeric(trend), 

                            nbias = nbias,
                            mortdata_ind = 1, remdata_ind = 1,
                            incdata_ind = incdata_ind, 
                            prevdata_ind = prevdata_ind,

                            inc_prior = inc_prior, cf_prior = cf_prior, rem_prior = rem_prior,

                            scf_isfixed = hp["scf","isfixed"],
                            sinc_isfixed = hp["sinc","isfixed"], 
                            srem_isfixed = hp["srem","isfixed"], 
                            lambda_cf_fixed = as.numeric(hp["scf","vals"]), 
                            lambda_inc_fixed = as.numeric(hp["sinc","vals"]), 
                            lambda_rem_fixed = as.numeric(hp["srem","vals"]), 

                            X=X, K=ncol(X),

                            sprior=sprior, eqage=eqage))
    if (trend) {
      datstans$inc_trend <- inc_trend
      datstans$cf_trend <- cf_trend
      datstans$nyr <- nage
      
    } else {
      datstans$inc_trend <- array(1, dim=c(nage,1))
      datstans$cf_trend <-  array(1, dim=c(nage,1)) 
      datstans$nyr <- 1
    }
    initsc <- initlist_const(initrates, cf_smooth, inc_smooth, remission, rem_smooth, 
                             eqage, smooth_inc, smooth_cf, const_cf, increasing_cf,
                             smooth_rem, const_rem, nbias,
                             hp["scf","isfixed"], hp["sinc","isfixed"], hp["srem","isfixed"])
    initsr <- initlist_random(nage, initrates, cf_smooth, inc_smooth, remission, rem_smooth, 
                              eqage, smooth_inc, smooth_cf, const_cf, increasing_cf,
                              smooth_rem, const_rem, nbias,
                              hp["scf","isfixed"], hp["sinc","isfixed"], hp["srem","isfixed"])
    if (method=="opt") { 
      opts <- rstan::optimizing(stanmodels$disbayes, data=datstans, init=initsc, draws=draws,
                                iter=iter,  importance_resampling=TRUE, ...)
      res <- list(fit=opts, fitu=initrates, method="opt")
    } else if (method=="vb"){ 
      fits <- rstan::vb(stanmodels$disbayes, data=datstans, init=initsr, ...)
      res <- list(fit=fits, fitu=initrates, method="vb")
    }
    else if (method=="mcmc") {
      fits <- rstan::sampling(stanmodels$disbayes, data=datstans, init=initsr, 
                              iter=iter, control=stan_control, ...)
      res <- list(fit=fits, fitu=initrates, method="mcmc")
    } else stop(sprintf("Unknown method: `%s`", method))
  } else {
    if (method=="opt") { 
      res <- list(fit = initrates$optu, method="opt")
    } else {
        fitu <- fit_unsmoothed(dat, inc_init=initrates$inc, rem_init=initrates$rem, 
                               mdata, idata, 
                               method = method,
                               iter = iter, stan_control = stan_control, 
                               ...)
      res <- list(fit=fitu, method="mcmc")
    }
  }
  res <- c(list(call=dbcall),
           res,
           list(nage=nage, narea=1, ng=1, dat=dat, stan_data=datstans, stan_inits=initsc,
                trend=trend))
  res$hp_fixed <- setNames(hp$vals, hp$pars)[hp$include & hp$isfixed]
  class(res) <- "disbayes"
  res
}

get_column <- function(data, str){
  col <- if (!is.null(str)) data[[str]] else NULL
  if (!is.null(str) & is.null(col))
    stop(sprintf("No column \"%s\" in data", str))
  col
}

check_age <- function(data, age="age", model="nonhier", area=NULL, gender=NULL) {
  if (!is.data.frame(data)) stop("`data` argument should be a data frame")
  if (is.null(data[[age]]))
    stop(sprintf("age variable `%s` not found in data", age))
  nage <- length(unique(data[[age]]))
  
  if (model=="hier") {
    narea <- length(unique(area))
    if (nrow(data) != nage * narea) {
      stop(sprintf("%s rows in data, expected n_ages x n_areas = %s x %s, where n_ages is number of unique ages in the data", nrow(data), nage, narea))
    }
    ## note data get ordered by area before calling this 
    if (!isTRUE(all.equal(data[[age]], rep(seq(from=0,to=nage-1), narea))))
      stop("data should be ordered by age in each area, with one value per year of age starting at age 0")
  }
  else if (model=="gender") {
    narea <- length(unique(area))
    ngender <- length(unique(gender))
    if (nrow(data) != nage * narea * ngender) {
      stop(sprintf("%s rows in data, expected n_ages x n_areas x n_genders = %s x %s x %s, where n_ages is number of unique ages, and n_genders is number of unique genders in data", nrow(data), nage, narea, ngender))
    }
    if (!isTRUE(all.equal(data[[age]], rep(seq(from=0,to=nage-1), narea*ngender))))
      stop("age variable should be ordered by age in each area and gender, with one value per year of age starting at age 0")
  }
  else if (model=="nonhier"){ 
    if (nrow(data) != nage)
      stop(sprintf("%s rows in data, but %s unique values in data[,age]. Should be one row per distinct year of age", nrow(data), nage))
    if (!isTRUE(all.equal(data[[age]], seq(from=0,to=nage-1))))
      stop("age variable should be ordered with one value per year of age starting at age 0")
  }
}

check_integer <- function(x, nd, prefix){
  badint <- which(x != floor(x))
  if (length(badint) > 0) {
    others <- if (length(badint) > 1) " and others" else ""
    stop(sprintf("%s_%s[%s]=%s should be integer", 
                 prefix, nd, badint[1], x[badint[1]]))
  }
}

check_binomial <- function(num, denom, prefix){
    badbinom <- which(num > denom)
    if (length(badbinom) > 0) {
        stop(sprintf("%s_num[%s]=%s should be <= %s_denom[%s]=%s", 
                     prefix, badbinom[1], num[badbinom[1]], prefix, badbinom[1], denom[badbinom[1]]))
    }
}

check_proportion <- function(x, prefix){
    badprop <- which(x < 0 | x > 1)
    if (length(badprop) > 0) {
        stop(sprintf("%s_est[%s]=%s should be in [0,1]", 
                     prefix, badprop[1], x[badprop[1]]))
    }
}

check_interval <- function(x, lower, upper, prefix){
    badint <- which(lower > upper)
    if (length(badint) > 0) {
        stop(sprintf("%s_lower[%s]=%s, should be < %s_upper[%s]=%s",
                     prefix, badint[1], lower[badint[1]], prefix, badint[1], upper[badint[1]]))
    }    
    badint <- which(x < lower | x > upper)
    if (length(badint) > 0) {
        stop(sprintf("%s_prob[%s]=%s should be inside the credible interval of (%s_lower[%s]=%s, %s_upper[%s]=%s)",
                     prefix, badint[1], x[badint[1]], 
                     prefix, badint[1], lower[badint[1]], 
                     prefix, badint[1], upper[badint[1]]))
    }
}

## Return either constant, num+denom, or error

## (a) If num and denom supplied then OK
## (b) If est and denom supplied then convert
## (c) If est, lower and upper supplied then convert 
## Else return error 

process_data <- function(data, prefix, num_str, denom_str, est_str, lower_str, upper_str, 
                         nage, ngroup=1, ngender=1, hier=FALSE){
  num <- get_column(data, num_str)
  denom <- get_column(data, denom_str)
  est <- get_column(data, est_str)
  lower <- get_column(data, lower_str)
  upper <- get_column(data, upper_str)
  supplied <- FALSE
  if (!is.null(num) && !is.null(denom)){
    res <- list(num=num, denom=denom)
    check_integer(num, "num", prefix)
    check_integer(denom, "denom", prefix)
    check_binomial(num, denom, prefix)
    supplied <- TRUE
  }
  else if (!is.null(est) && !is.null(denom)) {
    check_proportion(est, prefix)
    check_integer(denom, "denom", prefix)
    num <- round(est*denom)
    res <- list(num=num, denom=denom)
    supplied <- TRUE
  }
  else if (!is.null(est) && !is.null(lower) && !is.null(upper)) {
    check_interval(est, lower, upper, prefix)
    res <- as.list(ci2num(est, lower, upper))
    supplied <- TRUE
  }
  else {
    pnames <- list(inc="incidence", prev="prevalence", mort="mortality", rem="remission")
    if (prefix %in% c("prev","rem","inc")){
        res <- list(num=rep(0,nage), denom=rep(0,nage))
        supplied <- FALSE
    }
    else stop(sprintf("Not enough information supplied to obtain numerator and denominator for %s.\nNeed either numerator and denominator, estimate and denominator, or estimate with lower and upper credible limit", pnames[[prefix]]))
  }
  if (prefix=="prev") res$num[1] <- 0  # prevalence at age zero assumed exactly 0
  if (hier)  # for hierarchical models
    for (i in c("num","denom"))
      res[[i]] <- array(res[[i]], dim=c(nage, ngroup, ngender))
  res$supplied <- supplied
  names(res) <- paste(prefix, names(res), sep="_")
  res
}

check_trenddata <- function(trend, nage){
  baddf <- FALSE 
  if (is.data.frame(trend)){
    trend <- as.matrix(trend)
    if (!is.numeric(trend)) baddf <- TRUE 
  }
  if (!is.matrix(trend) || baddf)
    stop("trends data should be a matrix or data frame of numerics")
  if (nrow(trend) != nage || ncol(trend) != nage) {
    stop(sprintf("trend matrix of dimension (%s,%s), should be (%s,%s) to match the number of ages",nrow(trend),ncol(trend),nage,nage))
  }
}

check_eqage <- function(eqage, eqagehi, nage) {
    if (!is.numeric(eqage) || (length(eqage) > 1))
        stop("`eqage` must be a single number")
    if ((eqage > nage) || (eqage < 0))
        stop(sprintf("eqage is %s, should be in [0, nage=%s]", eqage, nage))
    if (!is.null(eqagehi)){
        if (!is.numeric(eqagehi) || (length(eqagehi) > 1))
            stop("`eqagehi` must be a single number")
        if ((eqagehi > nage) || (eqage < 0))
            stop(sprintf("eqagehi is %s, should be in [0, nage=%s]", eqagehi, nage))
        if (eqage > eqagehi)
            stop(sprintf("eqage is %s, eqagehi is %s, should have eqage<eqagehi", eqage, eqagehi))
    }
}

check_rate_prior <- function(prior, ismissing, allowed, allowed_string, prefix) {
    if (!is.numeric(prior) || (length(prior) != 2))
        stop(sprintf("`%s_prior` should be a numeric vector of 2 elements", prefix))
    if (!ismissing && !allowed)
        warning(sprintf("Ignoring `%s_prior`, which is only relevant if `%s_model` is %s", prefix, prefix, allowed_string))
}

check_sprior <- function(sprior){
  snew <- c(inc=1, cf=1, rem=1)
  if (!is.numeric(sprior)) stop("`sprior` should be numeric")
  if (is.null(names(sprior))) {
    if (length(sprior) != 3) stop("If not named, `sprior` should be a vector of 3 elements")
    names(sprior) <- c("inc", "cf", "rem")
  } 
  for (i in names(sprior)) {
    snew[i] <- sprior[i]
  }
  snew
}

check_hp_fixed <- function(hp_fixed, parlist, inc_model, cf_model, rem_model, ng) {
    if (!is.null(hp_fixed) && !is.list(hp_fixed)) stop("`hp_fixed` should be a list or NULL")
    if (!is.null(hp_fixed)) {
        if (is.null(names(hp_fixed))) stop("`hp_fixed` should be a named list")
        for (i in seq_along(hp_fixed)){
            if (!(names(hp_fixed)[i] %in% parlist$pars))
                warning(sprintf("Ignoring hp_fixed[[\"%s\"]] which is not a parameter in the model", names(hp_fixed)[i]))
            if (!is.null(hp_fixed[[i]]) && !is.logical(hp_fixed[[i]]) && (!is.numeric(hp_fixed[[i]]) || (length(hp_fixed[[i]]) > 1)))
                stop(sprintf("hp_fixed[[\"%s\"]] should be TRUE, FALSE or a single number", names(hp_fixed)[i]))
        }
        if ((inc_model %in% c("indep")) && is.numeric(hp_fixed[["sinc"]]))
            warning(sprintf("Ignoring hp_fixed[[\"sinc\"]] since `inc_model=\"%s\"`", inc_model))
        if ((cf_model %in% c("indep","const")) && is.numeric(hp_fixed[["scf"]]))
            warning(sprintf("Ignoring hp_fixed[[\"scf\"]] since `cf_model=\"%s\"`", cf_model))
        if ((rem_model %in% c("indep","const")) && is.numeric(hp_fixed[["srem"]]))
            warning(sprintf("Ignoring hp_fixed[[\"srem\"]] since `rem_model=\"%s\"`", rem_model))
        if ((!cf_model %in% c("default")) && is.numeric(hp_fixed[["sd_slope"]]))
            warning(sprintf("Ignoring hp_fixed[[\"sd_slope\"]] since `cf_model=\"%s\"`", cf_model))
        if ((ng == 1) && is.numeric(hp_fixed[["scfmale"]]))
            warning(sprintf("Ignoring hp_fixed[[\"scfmale\"]] since no gender effect in model"))
    }
}

##' Quick and dirty plot of estimates from disbayes models against age
##'
##' Posterior medians and 95% credible intervals for a quantity of interest are plotted against year of age.
##'
##' @param x Object returned by \code{\link{disbayes}}
##'
##' @param variable Name of the variable of interest to plot against age, by default case fatality rates.
##'
##' @param ... Other arguments. Currently unused
##'
##' @return A \code{ggplot2} object that can be printed to show the plot, or customised by adding \code{geom}s.
##'
##' Better plots can be drawn by \code{tidy}ing the object returned by \code{disbayes}, and using \code{ggplot2} directly on the tidy data frame that this produces. See the vignette for examples. 
##' 
##' @export
plot.disbayes <- function(x, variable="cf", ...){
  var <- NULL 
  summcf <- tidy(x) %>%
    filter(var==variable)
  p <- ggplot2::ggplot(summcf, ggplot2::aes_string(x="age")) + 
    ggplot2::geom_pointrange(ggplot2::aes_(y=as.name("50%"),
                                           ymin=as.name("2.5%"),
                                           ymax=as.name("97.5%")))
  p
}


##' @export
print.disbayes <- function(x, ...){
  cat("Call:\n")
  dput(x$call)
  cat(sprintf("\nTo summarise parameter estimates, call tidy() on the fitted model object\n"))
}

##' @export
summary.disbayes <- function(object, ...){
  print(object)
}

## disbayes by MCMC 

test_that("loo, MCMC", {
  db1 <- disbayes(dat = ihdbristol,
                  inc_num = "inc_num", inc_denom = "inc_denom",
                  prev_num = "prev_num", prev_denom = "prev_denom",
                  mort_num = "mort_num", mort_denom = "mort_denom",
                  cf_model = "smooth",
                  eqage = 30, method="mcmc", chains=1, iter=100)
  
  db2 <- disbayes(dat = ihdbristol,
                  inc_num = "inc_num", inc_denom = "inc_denom",
                  prev_num = "prev_num", prev_denom = "prev_denom",
                  mort_num = "mort_num", mort_denom = "mort_denom",
                  cf_model = "const",
                  eqage = 30, method="mcmc", chains=1, iter=100)
  
  loo1 <- loo(db1)
  loo2 <- loo(db2)
  lc <- loo::loo_compare(loo1,loo2)
  if ("model" %in% colnames(lc)) {
    expect_true(
      lc[lc$model == "model1", "elpd_diff"] > lc[lc$model == "model2", "elpd_diff"]
    )
  } else {
    expect_true(lc["model1","elpd_diff"] > lc["model2","elpd_diff"])
  }
  
  ## Individual contributions  
  li <- loo_indiv(loo1)
  expect_s3_class(li, "data.frame")
  expect_equal(sum(li$elpd_loo), 
               loo1$estimates["elpd_loo","Estimate"])
  
  ## Aggregated over outcome type
  liagg <- loo_indiv(loo1, agg=TRUE)
  expect_equal(sum(liagg$elpd_loo), 
               loo1$estimates["elpd_loo","Estimate"])
})

## Hierarchical models 

test_that("loo, hierarchical",{
  db1 <- disbayes_hier(ihdmale, age="age", group="area", 
                       inc_num = "inc_num", inc_denom = "inc_denom",
                       prev_num = "prev_num", prev_denom = "prev_denom",
                       mort_num = "mort_num", mort_denom = "mort_denom",
                       hp_fixed = list(sd_int=1, sd_slope=1, scf=1, sinc=1),
                       method="opt", verbose=TRUE)
  
  db2 <- disbayes_hier(ihdmale, age="age", group="area", 
                       inc_num = "inc_num", inc_denom = "inc_denom",
                       prev_num = "prev_num", prev_denom = "prev_denom",
                       mort_num = "mort_num", mort_denom = "mort_denom",
                       cf_model = "const", 
                       hp_fixed = list(sd_int=1, sinc=1),
                       method="opt", verbose=TRUE)
  
  loo1 <- loo(db1)
  loo2 <- loo(db2)
  lc <- loo::loo_compare(loo1,loo2)
  if ("model" %in% colnames(lc)) {
    expect_true(
      lc[lc$model == "model1", "elpd_diff"] > lc[lc$model == "model2", "elpd_diff"]
    )
  } else {
    expect_true(lc["model1","elpd_diff"] > lc["model2","elpd_diff"])
  }
  
  li <- loo_indiv(loo1)
  expect_equal(table(li$var)[[1]], 1700)
})

test_that("remission supplied, rem_model settings",{
  set.seed(1)
  db <- disbayes(dat = ihdbristol,
                 inc_num = "inc_num", inc_denom = "inc_denom",
                 prev_num = "prev_num", prev_denom = "prev_denom",
                 mort_num = "mort_num", mort_denom = "mort_denom",
                 rem_num = "rem_num", rem_denom = "rem_denom",
                 eqage = 40)
  res <- tidy(db)
  rem <- res %>% filter(var=="rem") %>% slice(1) %>% pull(mode)
  expect_equal(rem, 0.1, tol=1e-01)
  db <- disbayes(dat = ihdbristol,
                 inc_num = "inc_num", inc_denom = "inc_denom",
                 prev_num = "prev_num", prev_denom = "prev_denom",
                 mort_num = "mort_num", mort_denom = "mort_denom",
                 rem_num = "rem_num", rem_denom = "rem_denom",
                 rem_model = "smooth",
                 eqage = 40)
  res <- tidy(db)
  rem <- res %>% filter(var=="rem") %>% slice(10) %>% pull(mode)
  expect_equal(rem, 0.1, tol=1e-01)
  db <- disbayes(dat = ihdbristol,
                 inc_num = "inc_num", inc_denom = "inc_denom",
                 prev_num = "prev_num", prev_denom = "prev_denom",
                 mort_num = "mort_num", mort_denom = "mort_denom",
                 rem_num = "rem_num", rem_denom = "rem_denom",
                 rem_model = "indep",
                 eqage = 40)
  res <- tidy(db)
  rem <- res %>% filter(var=="rem") %>% head(100) %>% pull(mode)
  expect_equal(mean(rem), 0.1, tol=1e-01)
})

test_that("prevalence at age zero",{
  db <- disbayes(dat = ihdbristol,
                 inc_num = "inc_num", inc_denom = "inc_denom",
                 prev_num = "prev_num", prev_denom = "prev_denom",
                 mort_num = "mort_num", mort_denom = "mort_denom",
                 prev_zero = FALSE,
                 eqage = 40)
  res <- tidy(db)
  prevzero <- res %>% filter(age==0, var=="prev")
  expect_equal(prevzero$mode, 0)
  db <- disbayes(dat = ihdbristol,
                 inc_num = "inc_num", inc_denom = "inc_denom",
                 prev_num = "prev_num", prev_denom = "prev_denom",
                 mort_num = "mort_num", mort_denom = "mort_denom",
                 prev_zero = TRUE,
                 eqage = 40)
  res <- tidy(db)
  prevzero <- res %>% filter(age==0, var=="prev")
  expect_false(prevzero$mode == 0)
})


test_that("priors on rates",{
  db1 <- disbayes(dat = ihdbristol,
                  inc_num = "inc_num", inc_denom = "inc_denom",
                  prev_num = "prev_num", prev_denom = "prev_denom",
                  mort_num = "mort_num", mort_denom = "mort_denom",
                  cf_model = "const", cf_prior = c(1, 2),
                  eqage = 40)
  db2 <- disbayes(dat = ihdbristol,
                  inc_num = "inc_num", inc_denom = "inc_denom",
                  prev_num = "prev_num", prev_denom = "prev_denom",
                  mort_num = "mort_num", mort_denom = "mort_denom",
                  cf_model = "const", cf_prior = c(10, 2),
                  eqage = 40)
  cf1 <- tidy(db1) %>% filter(var=="cf", age==60) %>% pull(mode)
  cf2 <- tidy(db2) %>% filter(var=="cf", age==60) %>% pull(mode)
  expect_true(cf2 > cf1)
  
  
  db1 <- disbayes(dat = ihdbristol,
                  inc_num = "inc_num", inc_denom = "inc_denom",
                  prev_num = "prev_num", prev_denom = "prev_denom",
                  mort_num = "mort_num", mort_denom = "mort_denom",
                  rem_num = "rem_num", rem_denom = "rem_denom",
                  rem_model = "const", rem_prior = c(1, 2),
                  eqage = 40)
  db2 <- disbayes(dat = ihdbristol,
                  inc_num = "inc_num", inc_denom = "inc_denom",
                  prev_num = "prev_num", prev_denom = "prev_denom",
                  mort_num = "mort_num", mort_denom = "mort_denom",
                  rem_num = "rem_num", rem_denom = "rem_denom",
                  rem_model = "const", rem_prior = c(10, 2),
                  eqage = 40)
  rem1 <- tidy(db1) %>% filter(var=="rem", age==60) %>% pull(mode)
  rem2 <- tidy(db2) %>% filter(var=="rem", age==60) %>% pull(mode)
  expect_true(rem2 > rem1)
  
  set.seed(1)
  ## Zero incidence causes fuzz problems for inc_prob, so add 1
  inc_dat <- ihdbristol
  inc_dat$inc_num <- inc_dat$inc_num + 1
  db1 <- disbayes(dat = inc_dat,
                  inc_num = "inc_num", inc_denom = "inc_denom",
                  prev_num = "prev_num", prev_denom = "prev_denom",
                  mort_num = "mort_num", mort_denom = "mort_denom",
                  inc_model = "indep", inc_prior = c(1, 1.1),
                  eqage = 40)
  db2 <- disbayes(dat = inc_dat,
                  inc_num = "inc_num", inc_denom = "inc_denom",
                  prev_num = "prev_num", prev_denom = "prev_denom",
                  mort_num = "mort_num", mort_denom = "mort_denom",
                  inc_model = "indep", inc_prior = c(10, 1.1),
                  eqage = 40)
  inc1 <- tidy(db1) %>% filter(var=="inc", age==60) %>% pull(mode)
  inc2 <- tidy(db2) %>% filter(var=="inc", age==60) %>% pull(mode)
  expect_true(inc2 > inc1)
})

source("data.r")

test_that("standard disbayes model, MCMC",{
  dbres <- disbayes(dat = ihdbristol,
                    inc_num = "inc_num", inc_denom = "inc_denom",
                    prev_num = "prev_num", prev_denom = "prev_denom",
                    mort_num = "mort_num", mort_denom = "mort_denom",
                    eqage = 40, method="mcmc", algorithm="Fixed_param", 
                    chains=1, iter=100)
  expect_s3_class(dbres, "disbayes")
})

test_that("data supplied as estimate and denominator",{
  dbres <- disbayes(dat = ihdbristol,
                    inc_num = "inc_num", inc_denom = "inc_denom",
                    prev_num = "prev_num", prev_denom = "prev_denom",
                    mort_prob = "mort_prob", mort_denom = "mort_denom",
                    eqage = 40, method="mcmc", algorithm="Fixed_param", 
                    chains=1, iter=100)
  expect_s3_class(dbres, "disbayes")
})

test_that("data supplied as estimate and credible limits",{
  dbres <- disbayes(dat = ihdbristol,
                    inc_num = "inc_num", inc_denom = "inc_denom",
                    prev_num = "prev_num", prev_denom = "prev_denom",
                    mort_prob = "mort_prob", mort_lower = "mort_lower", mort_upper="mort_upper",
                    eqage = 40, method="mcmc", algorithm="Fixed_param", chains=1, iter=100)
  expect_s3_class(dbres, "disbayes")
})

test_that("errors when insufficient data supplied",{
  expect_error(
    disbayes(dat = ihdbristol,
             inc_num = "inc_num", inc_denom = "inc_denom",
             prev_num = "prev_num", prev_denom = "prev_denom",
             mort_prob = "mort_prob", 
             eqage = 40, algorithm="Fixed_param"),
    "Not enough information"
  )
  expect_error(disbayes(dat = ihdbristol,
           inc_num = "inc_num", inc_denom = "inc_denom",
           prev_num = "prev_num", prev_denom = "prev_denom",
           eqage = 40, algorithm="Fixed_param"),
           "Not enough information")
})

test_that("increasing case fatality",{
  dbres <- disbayes(dat = ihdbristol,
                    inc_num = "inc_num", inc_denom = "inc_denom",
                    prev_num = "prev_num", prev_denom = "prev_denom",
                    mort_num = "mort_num", mort_denom = "mort_denom",
                    cf_model = "increasing", 
                    eqage = 40,
                    chains = 1, iter=100, 
                    method="mcmc", algorithm="Fixed_param")
  expect_s3_class(dbres, "disbayes")
})

test_that("constant case fatality",{
  dbres <- disbayes(dat = ihdbristol,
                    inc_num = "inc_num", inc_denom = "inc_denom",
                    prev_num = "prev_num", prev_denom = "prev_denom",
                    mort_num = "mort_num", mort_denom = "mort_denom",
                    cf_model = "const", 
                    eqage = 40,
                    chains = 1, iter=100, 
                    method="mcmc", algorithm="Fixed_param")
  expect_s3_class(dbres, "disbayes")
})

test_that("unsmooth incidence", { 
  dbres <- disbayes(dat = ihdbristol, 
                    inc_num = "inc_num", inc_denom = "inc_denom",
                    prev_num = "prev_num", prev_denom = "prev_denom",
                    mort_num = "mort_num", mort_denom = "mort_denom",
                    inc_model = "indep", 
                    chains = 1, iter=100, 
                    method="mcmc", algorithm="Fixed_param")
  expect_s3_class(dbres, "disbayes")
})

#test_that("remission", {
  ## data not supplied 
  ### data supplied 
#})

test_that("errors in age structure", {
  ihdbristol$badage <- ihdbristol$age
  ihdbristol$badage[20:30] <- 92
  expect_error(disbayes(dat = ihdbristol, age = "nonexistent",
                        inc_num = "inc_num", inc_denom = "inc_denom",
                        prev_num = "prev_num", prev_denom = "prev_denom",
                        mort_num = "mort_num", mort_denom = "mort_denom",
                        ),
               "age variable `nonexistent` not found")
  expect_error(disbayes(dat = ihdbristol, age = "badage",
                        inc_num = "inc_num", inc_denom = "inc_denom",
                        prev_num = "prev_num", prev_denom = "prev_denom",
                        mort_num = "mort_num", mort_denom = "mort_denom"
                        ),
               "one row per distinct year of age")
  ihdbristol$badage <- ihdbristol$age
  ihdbristol$badage[20:21] <- 20:19
  expect_error(disbayes(dat = ihdbristol, age = "badage",
                        inc_num = "inc_num", inc_denom = "inc_denom",
                        prev_num = "prev_num", prev_denom = "prev_denom",
                        mort_num = "mort_num", mort_denom = "mort_denom"
                        ),
               "ordered with one value per year of age")
})


trends_inc <- ihdtrends %>% 
  filter(outcome=="Incidence", gender=="Female") %>%
  pivot_wider(names_from="year", values_from="p2017") %>%
  select(-age, -gender, -outcome) %>% 
  as.matrix()


trends_cf <- ihdtrends %>% 
  filter(outcome=="Case fatality", gender=="Female") %>%
  pivot_wider(names_from="year", values_from="p2017") %>%
  select(-age, -gender, -outcome) %>% 
  as.matrix()


## Could move this to tests/slow if causes a problem.  takes about 10 sec  

test_that("disbayes model with trends",{
  dbres <- disbayes(dat = ihdbristol,
                    inc_num = "inc_num", inc_denom = "inc_denom",
                    prev_num = "prev_num", prev_denom = "prev_denom",
                    mort_num = "mort_num", mort_denom = "mort_denom",
                    eqage = 40, method="mcmc", algorithm="Fixed_param", 
                    inc_trend = trends_inc, 
                    cf_trend = trends_cf, 
                    chains=1, iter=10)
  expect_s3_class(dbres, "disbayes")
})


test_that("errors in trend data", {
  trendsbad <- trends_inc[,1:2]
  expect_error(
    disbayes(data = ihdbristol,
             inc_num = "inc_num", inc_denom = "inc_denom",
             prev_num = "prev_num", prev_denom = "prev_denom",
             mort_num = "mort_num", mort_denom = "mort_denom",
             inc_trend = trendsbad),
    "trend matrix of dimension")
  
  expect_error(
    disbayes(data = ihdbristol,
             inc_num = "inc_num", inc_denom = "inc_denom",
             prev_num = "prev_num", prev_denom = "prev_denom",
             mort_num = "mort_num", mort_denom = "mort_denom",
             inc_trend = "wibble"),
    "trends data should be")
  
})


test_that("errors when data are invalid",{
  baddat <- ihdbristol
  baddat$inc_num <- baddat$inc_denom + 1
  expect_error(
    disbayes(dat = baddat,
             inc_num = "inc_num", inc_denom = "inc_denom",
             prev_num = "prev_num", prev_denom = "prev_denom",
             mort_prob = "mort_prob", 
             eqage = 40),
    "should be <="
  )
  baddat <- ihdbristol
  baddat$inc_num[40] <- baddat$inc_denom[40] - 0.001
  expect_error(
    disbayes(dat = baddat,
             inc_num = "inc_num", inc_denom = "inc_denom",
             prev_num = "prev_num", prev_denom = "prev_denom",
             mort_num = "mort_num", mort_denom = "mort_denom",
             eqage = 40)
    ,
    "should be integer"
  )
  baddat <- ihdbristol
  baddat$mort_lower[80] <- 0.01
  baddat$mort_prob[80] <- 0.001
  expect_error(
    disbayes(dat = baddat,
             inc_num = "inc_num", inc_denom = "inc_denom",
             prev_num = "prev_num", prev_denom = "prev_denom",
             mort_prob = "mort_prob", mort_lower = "mort_lower", mort_upper = "mort_upper",
             eqage = 40)
    ,
    "should be inside the credible interval"
  )
  baddat <- ihdbristol
  baddat$mort_lower[40] <- 0.1
  baddat$mort_upper[40] <- 0.001
  expect_error(
    disbayes(dat = baddat,
             inc_num = "inc_num", inc_denom = "inc_denom",
             prev_num = "prev_num", prev_denom = "prev_denom",
             mort_prob = "mort_prob", mort_lower = "mort_lower", mort_upper = "mort_upper",
             eqage = 40)
    ,
    "should be < mort_upper"
  )
})

test_that("sprior",{
  expect_error(disbayes(dat = ihdbristol,
           inc_num = "inc_num", inc_denom = "inc_denom",
           prev_num = "prev_num", prev_denom = "prev_denom",
           mort_num = "mort_num", mort_denom = "mort_denom",
           sprior = c(1,1)), "should be a vector of 3")
 db <- disbayes(dat = ihdbristol,
           inc_num = "inc_num", inc_denom = "inc_denom",
           prev_num = "prev_num", prev_denom = "prev_denom",
           mort_num = "mort_num", mort_denom = "mort_denom",
           sprior = c(cf=1, inc=1000))
 tidy(db) %>% filter(grepl("lambda", var))
 plot(db, var="inc")
})


test_that("errors and warnings in model specification",{
  expect_warning(
    disbayes(dat = ihdbristol,
             inc_num = "inc_num", inc_denom = "inc_denom",
             prev_num = "prev_num", prev_denom = "prev_denom",
             mort_num = "mort_num", mort_denom = "mort_denom",
             inc_prior = c(2,2)), "Ignoring `inc_prior`")
  expect_warning(
    disbayes(dat = ihdbristol,
             inc_num = "inc_num", inc_denom = "inc_denom",
             prev_num = "prev_num", prev_denom = "prev_denom",
             mort_num = "mort_num", mort_denom = "mort_denom",
             cf_model = "smooth",
             cf_prior = c(2,2)), "Ignoring `cf_prior`")
  expect_error(
    disbayes(dat = ihdbristol,
             inc_num = "inc_num", inc_denom = "inc_denom",
             prev_num = "prev_num", prev_denom = "prev_denom",
             mort_num = "mort_num", mort_denom = "mort_denom",
             inc_model = "indep",
             inc_prior = c(2,2,3)), "`inc_prior` should be a numeric vector of 2 elements")
  expect_warning(
    disbayes(dat = ihdbristol,
             inc_num = "inc_num", inc_denom = "inc_denom",
             prev_num = "prev_num", prev_denom = "prev_denom",
             mort_num = "mort_num", mort_denom = "mort_denom",
             rem_model = "const",
             hp_fixed = list(srem = 1)), "Ignoring hp_fixed")
  expect_error(
    disbayes(dat = ihdbristol,
             inc_num = "inc_num", inc_denom = "inc_denom",
             prev_num = "prev_num", prev_denom = "prev_denom",
             mort_num = "mort_num", mort_denom = "mort_denom",
             rem_model = "const",
             hp_fixed = list(sinc = "one")),
    'should be TRUE, FALSE or a single number')
  expect_error(
    disbayes(dat = ihdbristol,
             inc_num = "inc_num", inc_denom = "inc_denom",
             prev_num = "prev_num", prev_denom = "prev_denom",
             mort_num = "mort_num", mort_denom = "mort_denom",
             rem_model = "const",
             hp_fixed = list(sinc = c(1,1))),
    'should be TRUE, FALSE or a single number')
  expect_error(
    disbayes(dat = ihdbristol,
             inc_num = "inc_num", inc_denom = "inc_denom",
             prev_num = "prev_num", prev_denom = "prev_denom",
             mort_num = "mort_num", mort_denom = "mort_denom",
             rem_model = "const",
             eqage = 101),
    'eqage.+ should be')
  expect_error(
    disbayes(dat = ihdbristol,
             inc_num = "inc_num", inc_denom = "inc_denom",
             prev_num = "prev_num", prev_denom = "prev_denom",
             mort_num = "mort_num", mort_denom = "mort_denom",
             rem_model = "const",
             eqagehi = 101),
    'eqagehi.+ should be')
  expect_error(
    disbayes(dat = ihdbristol,
             inc_num = "inc_num", inc_denom = "inc_denom",
             prev_num = "prev_num", prev_denom = "prev_denom",
             mort_num = "mort_num", mort_denom = "mort_denom",
             rem_model = "const",
             eqage = 50 , eqagehi=40),
    'should have eqage<eqagehi')
})

test_that("hierarchical model with additive area and gender effects",{
  db <- disbayes_hier(ihdengland, age="age", group="area", gender="gender",
                      inc_num = "inc_num", inc_denom = "inc_denom",
                      prev_num = "prev_num", prev_denom = "prev_denom",
                      mort_num = "mort_num", mort_denom = "mort_denom",
                      iter=10, chains=1, method="mcmc", algorithm="Fixed_param")
  expect_s3_class(db, "disbayes_hier")
})

test_that("fixed smoothness for gender effect",{
  set.seed(1)
  db <- disbayes_hier(ihdengland, age="age", group="area", gender="gender",
                      inc_num = "inc_num", inc_denom = "inc_denom",
                      prev_num = "prev_num", prev_denom = "prev_denom",
                      mort_num = "mort_num", mort_denom = "mort_denom",
                      hp_fixed = list(scfmale = TRUE),
                      iter=10, chains=1, method="mcmc", algorithm="Fixed_param")
  expect_equal(db$hp_fixed[["scfmale"]],  0.5026838, tol=1e-01)
})

test_that("errors in specifying gender model", {
  expect_error(disbayes_hier(ihdengland, age="age", group="area",  
                             inc_num = "inc_num", inc_denom = "inc_denom",
                             prev_num = "prev_num", prev_denom = "prev_denom",
                             mort_num = "mort_num", mort_denom = "mort_denom"),
               "rows in data")
  
  expect_error(disbayes_hier(ihdengland %>% 
                               dplyr::filter(gender=="Female"), 
                             age="age", group="area", gender="gender",
                             inc_num = "inc_num", inc_denom = "inc_denom",
                             prev_num = "prev_num", prev_denom = "prev_denom",
                             mort_num = "mort_num", mort_denom = "mort_denom"),
               "Only one gender found in data")
  
  ## call nonhier model by mistake 
  expect_error(
    disbayes(ihdengland, age="age", 
             inc_num = "inc_num", inc_denom = "inc_denom",
             prev_num = "prev_num", prev_denom = "prev_denom",
             mort_num = "mort_num", mort_denom = "mort_denom"),
    "rows in data")
  ihdengland$badage <- ihdengland$age
  ihdengland$badage[20:30] <- 92
  
  expect_error(disbayes_hier(ihdengland, age="nonexistent", group="area", gender="gender",
                             inc_num = "inc_num", inc_denom = "inc_denom",
                             prev_num = "prev_num", prev_denom = "prev_denom",
                             mort_num = "mort_num", mort_denom = "mort_denom"),
               "age variable")
  
  expect_error(disbayes_hier(ihdengland, age = "badage", gender="gender",
                             inc_num = "inc_num", inc_denom = "inc_denom",
                             prev_num = "prev_num", prev_denom = "prev_denom",
                             mort_num = "mort_num", mort_denom = "mort_denom"
  ),
  "`group` variable not supplied")
  
  expect_error(disbayes_hier(ihdengland, age = "badage", group="area", gender="gender",
                             inc_num = "inc_num", inc_denom = "inc_denom",
                             prev_num = "prev_num", prev_denom = "prev_denom",
                             mort_num = "mort_num", mort_denom = "mort_denom"
  ),
  "value per year of age")
  
  ihdengland$badage <- ihdengland$age
  ihdengland$badage[20:21] <- 20:19
  expect_error(disbayes_hier(data = ihdengland, age = "badage", group="area", gender="gender",
                             inc_num = "inc_num", inc_denom = "inc_denom",
                             prev_num = "prev_num", prev_denom = "prev_denom",
                             mort_num = "mort_num", mort_denom = "mort_denom"
  ),
  "one value per year of age")
})

source("data.r")

test_that("fixing smoothness parameters",{
  dbres <- disbayes(dat = ihdbristol,
                    inc_num = "inc_num", inc_denom = "inc_denom",
                    prev_num = "prev_num", prev_denom = "prev_denom",
                    mort_num = "mort_num", mort_denom = "mort_denom",
                    hp_fixed = list(scf = 1.234), 
                    eqage = 40, method="mcmc", algorithm="Fixed_param",
                    chains=1, iter=100)
  expect_equal(dbres$hp_fixed[["scf"]], 1.234)
  
  set.seed(1)
  dbres <- disbayes(dat = ihdbristol,
                    inc_num = "inc_num", inc_denom = "inc_denom",
                    prev_num = "prev_num", prev_denom = "prev_denom",
                    mort_num = "mort_num", mort_denom = "mort_denom",
                    hp_fixed = list(scf = 1.234, sinc=TRUE), 
                    eqage = 40, method="mcmc", algorithm="Fixed_param",
                    chains=1, iter=100)
  expect_equal(dbres$hp_fixed[["scf"]], 1.234)
# can't get the results repeatable on different platforms, even setting the seed 
#  expect_equal(dbres$hp_fixed[["sinc"]], 5.822661, tol=1e-03)
  expect_equal(dbres$hp_fixed[["sinc"]], dbres$stan_data$lambda_inc_fixed, tol=1e-03)

  set.seed(1)
  dbres <- disbayes(dat = ihdbristol,
                    inc_num = "inc_num", inc_denom = "inc_denom",
                    prev_num = "prev_num", prev_denom = "prev_denom",
                    mort_num = "mort_num", mort_denom = "mort_denom",
                    rem_num = "rem_num", rem_denom = "rem_denom",
                    rem_model = "smooth",
                    hp_fixed = list(scf = 1.234, srem=TRUE), 
                    eqage = 40, method="mcmc", algorithm="Fixed_param",
                    chains=1, iter=100)
  expect_equal(dbres$hp_fixed[["scf"]], 1.234)
#  expect_equal(dbres$hp_fixed[["srem"]], 0.9352172, tol=1e-03)
  expect_equal(dbres$hp_fixed[["srem"]], dbres$stan_data$lambda_rem_fixed, tol=1e-03)
})

source("data.r")

test_that("standard disbayes model, variational Bayes",{
  dbres <- disbayes(dat = ihdbristol,
                    inc_num = "inc_num", inc_denom = "inc_denom",
                    prev_num = "prev_num", prev_denom = "prev_denom",
                    mort_num = "mort_num", mort_denom = "mort_denom",
                    eqage = 40, loo=FALSE, method="vb", algorithm="meanfield")
  expect_s3_class(dbres, "disbayes")
  ## Needs more experience to interpret diagnostic values, e.g. khat
})

source("data.r")

test_that("hierarchical model", {
  db <- disbayes_hier(ihdmale, age="age", group="area", 
                             inc_num = "inc_num", inc_denom = "inc_denom",
                           prev_num = "prev_num", prev_denom = "prev_denom",
                           mort_num = "mort_num", mort_denom = "mort_denom",
                    iter=10, chains=1, method="mcmc", algorithm="Fixed_param")
  expect_s3_class(db, "disbayes_hier")
})

test_that("errors in specifying hierarchical model", {
  expect_error(disbayes_hier(ihdengland, age="age", group="area", 
                             inc_num = "inc_num", inc_denom = "inc_denom",
                             prev_num = "prev_num", prev_denom = "prev_denom",
                             mort_num = "mort_num", mort_denom = "mort_denom"),
               "rows in data, expected")
  
  ## call nonhier model by mistake 
  expect_error(
    disbayes(ihdmale, age="age", 
             inc_num = "inc_num", inc_denom = "inc_denom",
             prev_num = "prev_num", prev_denom = "prev_denom",
             mort_num = "mort_num", mort_denom = "mort_denom"),
    "rows in data")
  ihdmale$badage <- ihdmale$age
  ihdmale$badage[20:30] <- 92
  
  expect_error(disbayes_hier(ihdmale, age="nonexistent", group="area", 
                             inc_num = "inc_num", inc_denom = "inc_denom",
                             prev_num = "prev_num", prev_denom = "prev_denom",
                             mort_num = "mort_num", mort_denom = "mort_denom"),
               "age variable")
  
  expect_error(disbayes_hier(ihdmale, age = "badage",
                             inc_num = "inc_num", inc_denom = "inc_denom",
                             prev_num = "prev_num", prev_denom = "prev_denom",
                             mort_num = "mort_num", mort_denom = "mort_denom"
  ),
  "`group` variable not supplied")
  
  expect_error(disbayes_hier(ihdmale, age = "badage", group="area",
                             inc_num = "inc_num", inc_denom = "inc_denom",
                             prev_num = "prev_num", prev_denom = "prev_denom",
                             mort_num = "mort_num", mort_denom = "mort_denom"
  ),
  "value per year of age")
  
  ihdmale$badage <- ihdmale$age
  ihdmale$badage[20:21] <- 20:19
  expect_error(disbayes_hier(data = ihdmale, age = "badage", group="area",
                             inc_num = "inc_num", inc_denom = "inc_denom",
                             prev_num = "prev_num", prev_denom = "prev_denom",
                             mort_num = "mort_num", mort_denom = "mort_denom"
  ),
  "one value per year of age")
})



test_that("fixed hyperparameters",{
  
  db <- disbayes_hier(ihdmale, age="age", group="area", 
                      inc_num = "inc_num", inc_denom = "inc_denom",
                      prev_num = "prev_num", prev_denom = "prev_denom",
                      mort_num = "mort_num", mort_denom = "mort_denom",
                      hp_fixed = list(scf = 1), 
                      iter=10, chains=1, method="mcmc", algorithm="Fixed_param")
  expect_equivalent(db$hp_fixed["scf"], 1)

  db <- disbayes_hier(ihdmale, age="age", group="area", 
                      inc_num = "inc_num", inc_denom = "inc_denom",
                      prev_num = "prev_num", prev_denom = "prev_denom",
                      mort_num = "mort_num", mort_denom = "mort_denom",
                      hp_fixed = list(scf = 1, sd_int = 1), 
                      iter=10, chains=1, method="mcmc", algorithm="Fixed_param")
  expect_equivalent(db$hp_fixed["sd_int"], 1)

  db <- disbayes_hier(ihdmale, age="age", group="area", 
                      inc_num = "inc_num", inc_denom = "inc_denom",
                      prev_num = "prev_num", prev_denom = "prev_denom",
                      mort_num = "mort_num", mort_denom = "mort_denom",
                      hp_fixed = list(scf = TRUE, sinc = 1.234, sd_int = TRUE), 
                      iter=10, chains=1, method="mcmc", algorithm="Fixed_param")
  expect_equivalent(db$hp_fixed["sinc"], 1.234)
  expect_type(db$hp_fixed["scf"], "double")
  
  expect_warning(
    disbayes_hier(ihdmale, age="age", group="area", 
                  inc_num = "inc_num", inc_denom = "inc_denom",
                  prev_num = "prev_num", prev_denom = "prev_denom",
                  mort_num = "mort_num", mort_denom = "mort_denom",
                  cf_model = "interceptonly",
                  hp_fixed = list(sd_slope = 1), 
                  chains=1, iter=10, method="mcmc", algorithm="Fixed_param"),
    "Ignoring hp_fixed")
  
})

##' Bayesian estimation of chronic disease epidemiology from incomplete data -
##' hierarchical model for case fatalities.  
##' 
##' A variant of \code{\link{disbayes}} in which data from different areas can be
##' related in a hierarchical model and, optionally, the effect of gender can be
##' treated as additive with the effect of area.  This is much more computationally 
##' intensive than the basic model in \code{\link{disbayes}}.  Time trends are not
##' supported in this function.
##'
##' @param group Variable in the data representing the area (or other grouping
##'   factor).
##'
##' @param gender If \code{NULL} (the default) then the data are one homogenous
##'   gender, and there should be one row per year of age.  Otherwise, set
##'   \code{gender} to a character string naming the variable in the data
##'   representing gender (or other categorical grouping factor).  Gender will then
##'   treated as a fixed additive effect, so the linear effect of gender on log
##'   case fatality is the same in each area.  The data should have one row per
##'   year of age and gender.
##'
##' @param cf_model The following alternative models for case fatality are
##'   supported:
##'
##'   \code{"default"} (the default). Random intercepts and slopes, and no
##'   further restriction.
##'
##'   \code{"interceptonly"}.  Random intercepts, but common slopes.
##'
##'   \code{"increasing"}. Case fatality is assumed to be an increasing function
##'   of age (note it is constant below \code{"eqage"} in all models) with a
##'   common slope for all groups.
##'   
##'   \code{"common"} Case fatality is an unconstrained function of age
##'   which is common to all areas, i.e. it has the same parameter values in
##'   every area.  This and \code{"increasing_common"} are used in situations
##'   where you want to compare a model with area-specific rates with a single model for
##'   the data aggregated over areas.  Modelling the area-disaggregated data using
##'   a common function for all areas is equivalent to a model for the aggregated data,
##'   and can be statistically compared (using cross-validation) with a model with
##'   area-specific rates. 
##' 
##'   \code{"increasing_common"} Case fatality is an increasing function of age
##'   which is common to all areas. 
##'
##'   \code{"const"} Case fatality is assumed to be constant with age, for all
##'   ages, but different in each area. 
##'
##'   \code{"const_common"} Case fatality is a constant over all ages and areas. 
##'
##' In all models, case fatality is a smooth function of age.
##'
##' @param inc_model Model for how incidence varies with age.
##'
##'   \code{"smooth"} (the default). Incidence is modelled as a smooth spline
##'   function of age, independently for each area (and gender).
##'
##'   \code{"indep"} Incidence rates for each year of age, area (and gender) are
##'   estimated independently.
##'
##' @param rem_model Model for how remission varies with age.  Currently
##'   supported models are \code{"const"} for a constant remission rate over all
##'   ages, \code{"const"} for a smooth spline,  or \code{"indep"} for a different remission rates estimated
##'   independently for each age with no smoothing.
##'
##' @param hp_fixed A list with one named element for each hyperparameter
##' to be fixed.  The value should be either 
##' 
##' * a number (to fix the hyperparameter at this number) 
##' 
##' * \code{TRUE} (to fix the hyperparameter at the posterior mode from a training run
##' where it is not fixed)
##' 
##' If the element is either \code{NULL}, \code{FALSE}, or omitted from the list, 
##' then the hyperparameter is given a prior and estimated as part of the Bayesian model.  
##' 
##' The hyperparameters that can be fixed are 
##' 
##' * \code{scf} Smoothness parameter for the spline relating case fatality to age.
##' 
##' * \code{sinc} Smoothness parameter for the spline relating incidence to age.  
##' 
##' * \code{scfmale} Smoothness parameter for the spline defining how the gender 
##' effect relates to age.  Only for models with additive gender and area effects. 
##' 
##' * \code{sd_int} Standard deviation of random intercepts for case fatality.
##' 
##' * \code{sd_slope} Standard deviation of random slopes for case fatality.
##' 
##' For example, to fix the case fatality smoothness to 1.2, fix the incidence
##' smoothness to its posterior mode, and estimate all the other hyperparameters, 
##' specify \code{hp_fixed = list(scf=1.2, sinc=TRUE)}.
##'
##' @param nfold_int_guess  Prior guess at the ratio of case fatality between a
##'   high risk (97.5% quantile) and low risk (2.5% quantile) area.
##'
##' @param nfold_int_upper  Prior upper 95% credible limit for the ratio in
##'   average case fatality between a high risk (97.5% quantile) and low risk
##'   (2.5% quantile) area.
##'
##' @param nfold_slope_guess,nfold_slope_upper This argument and the next
##'   argument define the prior distribution for the variance in the random
##'   linear effects of age on log case fatality.   They define a prior guess
##'   and upper 95% credible limit for the ratio of case fatality slopes
##'   between a high trend (97.5% quantile) and low risk (2.5% quantile) area.
##'   (Note that the model is not exactly linear, since departures from
##'   linearity are defined through a spline model.  See the Jackson et al.
##'   paper for details.).
##'
##' @param mean_int_prior Vector of two elements giving the prior mean and
##'   standard deviation respectively for the mean random intercept for log case
##'   fatality.
##'
##' @param mean_slope_prior Vector of two elements giving the prior mean and
##'   standard deviation respectively for the mean random slope for log case
##'   fatality.
##'
##' @param gender_int_priorsd Prior standard deviation for the additive effect
##'   of gender on log case fatality
##'
##' @param gender_slope_priorsd Prior standard deviation for the additive effect
##'   of gender on the linear age slope of log case fatality
##'
##' @inheritParams disbayes
##' 
##' @return  A list including the following components
##'
##'  \code{call}: Function call that was used. 
##'
##'  \code{fit}: An object containing posterior samples from the fitted model,
##'   in the \code{stanfit} format returned by the \code{\link[rstan]{stan}}
##'   function in the \pkg{rstan} package.
##'
##'  \code{method}:  Optimisation method that was chosen.
##'
##'  \code{nage}: Number of years of age in the data
##'  
##'  \code{narea}: Number of areas (or other grouping variable that defines the hierarchical model).
##'
##'  \code{ng}: Number of genders (or other categorical variable whose effect is treated as 
##'  additive with the area effect).
##'
##'  \code{groups}: Names of the areas (or other grouping variable), taken from the factor levels in the
##'  original data.
##'
##'  \code{genders}: Names of the genders (or other categorical variable), taken from the factor levels in the
##'  original data. 
##'  
##'  \code{dat}: A list containing the input data in the form of numerators
##'   and denominators.
##' 
##'  \code{stan_data}: Full list of data supplied to Stan
##' 
##'  \code{stan_inits}: Full list of parameter initial values supplied to Stan
##'
##'  \code{trend}: Whether a time trend was modelled
##'
##'  \code{hp_fixed} Values of any hyperparameters that are fixed during the main model fit. 
##'  
##' @references Jackson C, Zapata-Diomedi B, Woodcock J. 
##' "Bayesian multistate modelling of incomplete chronic disease burden data" 
##' \url{https://arxiv.org/abs/2111.14100}
##'
##' @md
##' 
##' @export
disbayes_hier <- function(data,
                          group, 
                          gender=NULL, 
                          inc_num=NULL, inc_denom=NULL, inc_prob=NULL, inc_lower=NULL, inc_upper=NULL,
                          prev_num=NULL, prev_denom=NULL, prev_prob=NULL, prev_lower=NULL, prev_upper=NULL,
                          mort_num=NULL, mort_denom=NULL, mort_prob=NULL, mort_lower=NULL, mort_upper=NULL,
                          rem_num=NULL, rem_denom=NULL, rem_prob=NULL, rem_lower=NULL, rem_upper=NULL,
                          age="age",
                          cf_init = 0.01,
                          eqage = 30,
                          eqagehi = NULL,
                          cf_model = "default",
                          inc_model = "smooth",
                          rem_model = "const",
                          prev_zero = FALSE, 
                          sprior = c(1, 1, 1),
                          hp_fixed = NULL,
                          nfold_int_guess = 5,  nfold_int_upper = 100,
                          nfold_slope_guess = 5, nfold_slope_upper = 100,
                          mean_int_prior = c(0,10), mean_slope_prior = c(5,5),
                          gender_int_priorsd = 0.82, gender_slope_priorsd = 0.82, 
                          inc_prior = c(1.1, 0.1), 
                          rem_prior = c(1.1, 1), 
                          method = "opt",
                          draws = 1000,
                          iter = 10000,
                          stan_control = NULL, 
                          ...
                          )
{
    dbcall <- match.call()
    cf_model <- match.arg(cf_model, c("default","interceptonly","increasing",
                                      "common", "increasing_common", "const", "const_common"))
    inc_model <- match.arg(inc_model, c("smooth", "indep"))
    rem_model <- match.arg(rem_model, c("smooth", "const", "indep"))
    const_rem <- (rem_model=="const")
    smooth_inc <- (inc_model=="smooth")
    data <- as.data.frame(data)
    
    if (missing(group)) stop("`group` variable not supplied")
    glevs <- levels(factor(data[,group]))
    area <- match(data[,group], glevs)
    narea <- length(unique(area))
    data <- data[order(area),]

    if (is.null(gender)) {
        ng <- 1
        check_age(data, age=age, model="hier", area=area)
        nage <- nrow(data[area==1,,drop=FALSE])
        genlevs <- NULL 
    } else { 
        genlevs <- levels(factor(data[,gender]))
        gender <- match(data[,gender], genlevs)
        ng <- length(unique(gender))
        if (ng == 1) stop("Only one gender found in data")
        ## order data so that vector reads to array(nage, narea, ng)
        ## leftmost subscript moves fastest, rightmost moves slowest 
        data <- data[order(gender,area),,drop=FALSE]
        check_age(data, age=age, model="gender", area=area, gender=gender)
        nage <- nrow(data[area==1 & gender==1,,drop=FALSE])
    }

    inc_data <- process_data(data, "inc", inc_num, inc_denom, inc_prob, inc_lower, inc_upper, nage, narea, ng, hier=TRUE)
    prev_data <- process_data(data, "prev", prev_num, prev_denom, prev_prob, prev_lower, prev_upper, nage, narea, ng, hier=TRUE)
    if (!inc_data$inc_supplied && !prev_data$prev_supplied)
        stop("At least one of incidence or prevalence should be supplied")
    mort_data <- process_data(data, "mort", mort_num, mort_denom, mort_prob, mort_lower, mort_upper, nage, narea, ng, hier=TRUE)
    if (!mort_data$mort_supplied)
        stop("Mortality data should be supplied")
    rem_data <- process_data(data, "rem", rem_num, rem_denom, rem_prob, rem_lower, rem_upper, nage, narea, ng, hier=TRUE)
    remission <- rem_data$rem_supplied
    smooth_rem <- (remission && rem_model=="smooth")

    dat <- c(inc_data, prev_data, mort_data, rem_data, nage=nage, remission=as.numeric(remission))
    dat$supplied <- NULL
    datagg <- hierdata_to_agg(dat, area)
    prev_zero <- prev_zero || (!is.null(dat$prev_num) && any(dat$prev_num[1,,] > 0))
    check_rate_prior(inc_prior, missing(inc_prior), inc_model %in% c("indep"), "\"indep\"", "inc")
    check_rate_prior(rem_prior, missing(rem_prior), rem_model %in% c("const","indep"), "\"const\" or \"indep\"", "rem")
    check_eqage(eqage, eqagehi, nage)
    mdata <- list(remission=remission, eqage=eqage, const_rem=const_rem,
                  smooth_rem=smooth_rem, prev_zero=prev_zero,
                  inc_prior=inc_prior, cf_prior=c(2,0.1), rem_prior=rem_prior)
    idata <- list(cf_init=cf_init) 
    sprior <- check_sprior(sprior)
    
    initrates <- init_rates(datagg, mdata, idata, ...)
    cf_smooth <- init_smooth(log(initrates$cf), eqage, eqagehi, s_opts=NULL)
    inc_smooth <- if (smooth_inc) init_smooth(log(initrates$inc), eqage, eqagehi, s_opts=NULL) else NULL
    rem_smooth <- if (smooth_rem) init_smooth(log(initrates$rem), eqage, eqagehi, s_opts=NULL) else NULL 

    beta_in <- cf_smooth$beta 
    betainc_in <- inc_smooth$beta 
    betarem_in <- rem_smooth$beta 
    lam_in <- laminc_in <- lamrem_in <- 0.5 
    K <- ncol(cf_smooth$X)

    interceptonly <- (cf_model=="interceptonly") 
    increasing <- (cf_model %in% c("increasing","increasing_common"))
    common <- (cf_model %in% c("common", "increasing_common","const_common"))
    const_cf <- (cf_model %in% c("const", "const_common"))

    ## Handle fixed hyperparameters and empirical Bayes estimation
    check_hp_fixed(hp_fixed, .disbayes_hier_hp, inc_model, cf_model, rem_model, ng=ng)
    if (inc_model %in% c("indep") && !is.null(hp_fixed[["sinc"]])) hp_fixed[["sinc"]] <- NULL
    if (cf_model %in% c("const") && !is.null(hp_fixed[["scf"]])) hp_fixed[["scf"]] <- NULL
    if (rem_model %in% c("const") && !is.null(hp_fixed[["srem"]])) hp_fixed[["srem"]] <- NULL
    if (ng==1 && !is.null(hp_fixed[["scfmale"]])) hp_fixed[["scfmale"]] <- NULL 
    if (!(cf_model %in% c("default")) && !is.null(hp_fixed[["sd_slope"]])) hp_fixed[["sd_slope"]] <- NULL
    hp <- eb_disbayes(.disbayes_hier_hp, hp_fixed, dbcall, disbayes_hier, method, list(...))
    if (ng==1) {
      hp$include[hp$pars=="scfmale"] <- FALSE
      hp["scfmale","isfixed"] <- TRUE
    }
    if (inc_model=="indep"){
      hp$include[hp$pars=="sinc"] <- FALSE
      hp["sinc","isfixed"] <- TRUE
    }
    
    inits_hier_fn <- function() { 
      inc_init  <- rlnorm(nage, meanlog=log(initrates$inc), sdlog=initrates$inc/10)
      rem_init  <- rlnorm(nage, meanlog=log(initrates$rem), sdlog=initrates$rem/10)
      beta_in <- rnorm(length(beta_in), beta_in, abs(beta_in)/10)
      betainc_in <- if (smooth_inc) rnorm(length(betainc_in), betainc_in, abs(betainc_in)/10) else numeric()
      betarem_in <- if (smooth_rem) rnorm(length(betarem_in), betarem_in, abs(betarem_in)/10) else numeric()
      sd_in <- rlnorm(1, meanlog=log(0.1), sdlog=0.01)
      lam_in <- rlnorm(length(lam_in), meanlog=log(lam_in), sdlog=lam_in/10)
      laminc_in <- rlnorm(length(laminc_in), meanlog=log(laminc_in), sdlog=laminc_in/10)
      lamrem_in <- rlnorm(length(lamrem_in), meanlog=log(lamrem_in), sdlog=lamrem_in/10)
      pz_in <- if (prev_zero) as.array(max(dat$prev_num[1],1)/max(dat$prev_denom[1],2)) else numeric()

      inits_hier <- list(prevzero = array(rep(pz_in, narea*prev_zero*ng),
                                          dim = c(narea*prev_zero, ng)),
                         inc_par = array(rep(initrates$inc, narea*ng),
                                         dim=c(nage*(1 - smooth_inc), narea, ng)), # remove if mandating SI
                         rem_par = array(rep(rem_init, ng*remission*(1-smooth_rem)), 
                                         dim=c(remission*(1-smooth_rem)*(nage*(1-const_rem) + 1*const_rem), ng)),
                         barea = matrix(0, nrow=(K-2)*(1-const_cf),
                                        ncol = narea*(1-common) + 1*common),
                         barea_slope = matrix(0, nrow=(1-interceptonly)*(1-const_cf),
                                              ncol=narea*(1-common) + 1*common),
                         barea_inter = matrix(0, nrow=1, ncol=narea*(1-common)),
                         bmale = if (ng==1) numeric() else rep(0, K),
                         beta_inc = array(rep(betainc_in, narea*ng),
                                          dim=c(K*smooth_inc, narea, ng)),
                         beta_rem = array(rep(betarem_in, narea*ng),
                                          dim=c(K*smooth_rem, narea, ng)),
                         lcfbase = if (increasing) as.array(rep(beta_in[K-1],
                                                       narea*(1-common) + 1*common)) else numeric(),
                         mean_inter = if (beta_in[K-1] > 0) beta_in[K] else mean(log(initrates$cf)),
                         mean_slope = if (const_cf) numeric() else as.array(beta_in[K-1]), 
                         sd_inter = if (hp["sd_int","isfixed"]) numeric() else as.array(sd_in),
                         sd_slope = if (interceptonly || increasing || const_cf || 
                                        hp["sd_slope","isfixed"]) numeric() else as.array(sd_in),
                         lambda_cf = if (const_cf || hp["scf","isfixed"]) numeric() else as.array(lam_in),
                         lambda_cf_male = if (ng==1  || hp["scfmale","isfixed"]) numeric() else as.array(lam_in), 
                         lambda_inc = if (!smooth_inc || hp["sinc","isfixed"]) numeric() else as.array(laminc_in),
                         lambda_rem = if (!smooth_rem || hp["srem","isfixed"]) numeric() else as.array(lamrem_in)
      )
      inits_hier
    }

    gpint <- gammapars_hier(nfold_int_guess, nfold_int_upper)
    gpslope <- gammapars_hier(nfold_slope_guess, nfold_slope_upper)

    datstans <- c(dat, list(interceptonly=interceptonly, 
                            increasing=increasing, 
                            common=common, 
                            narea=narea, 
                            ng=ng,
                            const_cf = as.numeric(const_cf), 
                            smooth_inc=as.numeric(smooth_inc),
                            smooth_rem=as.numeric(smooth_rem),
                            prev_zero=as.numeric(prev_zero),
                            eqage=eqage, 
                            X=cf_smooth$X, K=K, sprior=sprior,
                            mipm = mean_int_prior[1], 
                            mips = mean_int_prior[2], 
                            mism = mean_slope_prior[1], 
                            miss = mean_slope_prior[2], 
                            gpint_a = gpint["a"] + 2, gpint_b =  gpint["b"],
                            gpslope_a = gpslope["a"] + 2, gpslope_b =  gpslope["b"],
                            gender_int_priorsd = gender_int_priorsd,
                            gender_slope_priorsd = gender_slope_priorsd,
                            sd_int_isfixed = hp["sd_int","isfixed"],
                            sd_slope_isfixed = hp["sd_slope","isfixed"], 
                            sd_int_fixed = hp["sd_int","vals"], 
                            sd_slope_fixed = hp["sd_slope","vals"],
                            scf_isfixed = hp["scf","isfixed"],
                            scfmale_isfixed = hp["scfmale","isfixed"],
                            sinc_isfixed = hp["sinc","isfixed"], 
                            srem_isfixed = hp["srem","isfixed"], 
                            lambda_cf_fixed = as.numeric(hp["scf","vals"]), 
                            lambda_cf_male_fixed = as.numeric(hp["scfmale","vals"]), 
                            lambda_inc_fixed = as.numeric(hp["sinc","vals"]),
                            lambda_rem_fixed = as.numeric(hp["srem","vals"])
                            ))

        if (method=="opt") { 
            opts <- rstan::optimizing(stanmodels$disbayes_hier, data=datstans, init=inits_hier_fn, iter=iter, 
                                      draws=draws, importance_resampling=TRUE, ...)
          res <- list(fit=opts, method="opt")
        } else if (method=="vb"){
            fits <- rstan::vb(stanmodels$disbayes_hier, data=datstans, init=inits_hier_fn, iter=iter, ...)
            res <- list(fit=fits, method="vb")
        } else if (method=="mcmc"){
            fits <- rstan::sampling(stanmodels$disbayes_hier, data=datstans, init=inits_hier_fn, iter=iter, control=stan_control, ...)
            res <- list(fit=fits, method="mcmc")
        } else stop(sprintf("Unknown method: `%s`", method))
    res <- c(list(call=dbcall),
             res,
             list(nage=nage, narea=narea, ng=ng, groups=glevs, genders=genlevs, 
                  stan_data=datstans, stan_inits=inits_hier_fn()))
    res$hp_fixed <- setNames(hp$vals, hp$pars)[hp$include & hp$isfixed]
    class(res) <- c("disbayes_hier","disbayes")
    res
}

hierdata_to_agg <- function(dat, groups){
  for (i in c("inc_num", "inc_denom", "prev_num", "prev_denom", 
              "mort_num", "mort_denom", "rem_num",  "rem_denom"))
    dat[[i]] <- rowSums(dat[[i]][,,1])
  dat
}


# Returns the rate of the exponential distribution 
##  placed on the SDs of random intercepts or slopes in the hierarchical model 
# // (log(10) - log(1)) = 4.6 divided by 4 is SD (on log scale) 
# // representing a 10-fold variation in intercept or slope between 2.5 and 97.5 percentile area
# // Assume 95% chance that SD is less than this
# // satisfied by Exponential(rate 5.1)
# // qexp(p=0.95, (log(10) - log(1)) / (2*qnorm(0.975)))
# // qexp(p=0.99, (log(5) - log(1)) / (2*qnorm(0.975)))

# // Exponential now replaced by gamma 

# // posterior mean is 0.16, representing only 2-fold variation. 

## try putting something really tight on these and see what happens. 
## nfold=5 for intercepts, nfold=2 for slopes

hier_prior_var <- function(nfold=10, range = 0.95, confidence=0.95){
  alpha  <- (1  - range)/2
  qexp(p = confidence, rate = log(nfold) / (qnorm(1-alpha)  - qnorm(alpha))) 
}

##' @importFrom stats pgamma uniroot
##' 
gammasd_hier <- function(nfold_mean=5, nfold_upper=100){
  m <- log(nfold_mean) / 4
  fn <- function(s) { stats::pgamma(log(nfold_upper)/4, m^2/s^2, m/s^2) - 0.975 } 
  uniroot(fn, c(0.001, 3), extendInt="yes")$root
}

gammapars_hier <- function(nfold_mean=5, nfold_upper=100){
  m <- log(nfold_mean) / 4
  s <- gammasd_hier(nfold_mean, nfold_upper)
  c(a = (m/s)^2, 
    b = m/s^2)
}


## Gets vectors of indices for each dimension of an unrolled array 
##
## e.g.  array_indvecs(age = 2, area = 3, gender = 2)
##'
##' @importFrom stats setNames
##' 
array_indvecs <- function(...){
    args <- list(...)
    if (is.null(names(args))) names(args) <- paste0("V",seq_along(args))
  dnames <- names(args)
  dims <- unlist(args)
  z <- array(dim=dims)
  res <- vector(length(dims), mode="list")
  res <- stats::setNames(res, dnames)
  for (i in seq_along(res)){
    res[[i]] <- as.vector(slice.index(z, i))
  }
  as.data.frame(res)
}


##' Quick plot of estimates from hierarchical disbayes models against age
##'
##'
##' Posterior medians and 95\% credible intervals for a quantity of interest are plotted against year of age.  
##'
##' @param x Object returned by \code{\link{disbayes_hier}}
##'
##' @param variable Name of the variable of interest to plot against age, by default case fatality rates.
##'
##' @param ci Show 95\% credible intervals with ribbons.
##' 
##' @param ... Other arguments. Currently unused
##'
##' @return A \code{ggplot2} object that can be printed to show the plot, or
##' customised by adding \code{geom}s.
##'
##' Better plots can be drawn by \code{tidy}ing the object returned by \code{disbayes}, and using \code{ggplot2} directly on the tidy data frame that this produces.  See the vignette for examples. 
##' 
##' @export
plot.disbayes_hier <- function(x, variable="cf", ci=FALSE, ...){
    var <- gender <- NULL
    summ <- tidy(x) %>% 
      dplyr::filter(var==variable)
    p <- ggplot2::ggplot(summ, ggplot2::aes_string(x="age", 
                                                   group="area",
                                                   col="area"))
    if (!is.null(summ[["50%"]]))
      p <- p + ggplot2::geom_line(ggplot2::aes_string(y=summ$`50%`))
    else 
      p <- p + ggplot2::geom_line(ggplot2::aes_string(y=summ$mode))
    if (!is.null(summ[["2.5%"]]) && ci)
      p <- p + ggplot2::geom_ribbon(ggplot2::aes_string(
        ymin=summ$`2.5%`, 
        ymax=summ$`97.5%`, fill="area"))
    if (x$ng > 1) {
      p <- p + ggplot2::facet_grid(rows=vars(gender))
    }
    p
}

test_that("common case fatality to all areas", {
  db <- disbayes_hier(ihdmale, age="age", group="area", 
                      inc_num = "inc_num", inc_denom = "inc_denom",
                      prev_num = "prev_num", prev_denom = "prev_denom",
                      mort_num = "mort_num", mort_denom = "mort_denom",
                      cf_model = "increasing_common",
                      hp_fixed = list(scf = 1), 
                      method = "opt")
  plot(db)
  foo <- tidy(db)
  foo %>% filter(age==60, var=="cf")
  
})

##' Leave-one-out cross validation for disbayes models
##'
##' @param x A model fitted by \code{\link{disbayes}}.  Any of the computation methods
##' are supported. 
##'
##' @param outcome Either \code{"overall"}, to assess the fit to all data, or 
##' one of \code{"inc"}, \code{"prev"}, \code{"mort"} or \code{"rem"}, to assess the fit
##' to the incidence data, prevalence data, mortalidy data or remission data, respectively.
##'
##' @param ... Other arguments (currently unused).
##'
##' @return An object of class \code{"loo"} as defined by the \pkg{loo} package.
##'
##' @seealso \code{\link{loo_indiv}} to return tidied observation-specific contributions
##' to the overall model fit computed here. 
##'
##' @export
loo.disbayes <- function(x, outcome="overall", ...){
  if (x$method=="opt"){
    res <- loo_disbayes_opt(x, outcome=outcome)    
  } else {
    res <- loo_disbayes_mcmc(x, outcome=outcome)    
  }
  attr(res, "disbayes_info") <- list(hier = inherits(x, "disbayes_hier"))
  res
}

loo_disbayes_mcmc <- function(x, outcome="overall") { 
  ll <- loo::extract_log_lik(x$fit, sprintf("ll_%s",outcome), merge_chains=FALSE)
  r_eff <- loo::relative_eff(exp(ll))
  res <- loo::loo(ll, r_eff = r_eff)
  ## name the rows of the indiv-level contributions to match the outcome probs
  ## note in same order as Stan generated_quantities
  inames <- list()
  for (i in c("mort","inc","prev","rem")){
      supp <- x$dat[[sprintf("%s_supplied",i)]]
      if (is.null(supp) || isTRUE(supp))
          inames[[i]] <- grep(sprintf("%s_prob",i), names(x$fit), value=TRUE)
  }
  rownames(res$pointwise) <- as.vector(unlist(inames))
  res
}

loo_disbayes_opt <- function(x, outcome="overall") { 
  log_p <- x$fit$log_p # log density of the posterior.
  log_g <- x$fit$log_g # log density of the approximation
  draws <- x$fit$theta_tilde
  outs <- character()
  if (x$dat$inc_supplied) outs <- c(outs, "inc")
  if (x$dat$prev_supplied) outs <- c(outs, "prev")
  outs <- c(outs, "mort")
  if (x$dat$rem_supplied) outs <- c(outs, "rem")
  if (outcome=="overall") {
    outcome_names <- paste(rep(outs,each=2), rep(c("num","denom"), length(outs)), sep="_")
    datlist <- parlist <- vector(length(outs), mode="list")
    for (i in seq_along(outs)){
      num <- as.vector(x$stan_data[[sprintf("%s_num",outs[i])]])
      denom <- as.vector(x$stan_data[[sprintf("%s_denom",outs[i])]])
      datlist[[i]] <- data.frame(num, denom)
      outcome_regex <- sprintf("^(%s)_prob\\[.+\\]", outs[i])
      prob_names <- grepl(outcome_regex, colnames(draws))
      parlist[[i]] <- t(draws[,prob_names]) # nsam x nobs matrix
    }
    dat <- do.call("rbind", datlist)
    prob_draws <- do.call("rbind", parlist)
  } else if (outcome %in% outs) {
    num <- as.vector(x$stan_data[[sprintf("%s_num",outcome)]])
    denom <- as.vector(x$stan_data[[sprintf("%s_denom",outcome)]])
    dat <- data.frame(num, denom)
    prob_names <- grepl(sprintf("^%s_prob\\[.+\\]", outcome), colnames(draws))
    prob_draws <- t(draws[,prob_names]) # nsam x nobs matrix
  } else stop(sprintf("`outcome` should be one of \"overall\", %s"), 
              paste("\"outs\"", collapse=", "))
  ll_mat <- t(dbinom(x = dat[,"num"], size = dat[,"denom"], prob = prob_draws, log=TRUE))
  loo_ap <-
    loo::loo_approximate_posterior(
      x = ll_mat,
      draws = x$fit$theta_tilde,
      data = dat,
      log_p = log_p,
      log_g = log_g,
      cores = 1
    )
  loo_ap
}

##' Extract observation-specific contributions from a disbayes leave-one-out cross validation
##' 
##' @param x For \code{loo_indiv}, an object returned by \code{\link{loo.disbayes}}.   For \code{looi_disbayes}, an object returned by \code{\link{disbayes}}. 
##'
##' @param agg If \code{TRUE} then the observation-specific contributions are aggregated over
##' outcome type, returning a data frame with one row for each of incidence, prevalence, mortality
##' and remission (if remission is included in the model), and one column for each of \code{"elpd_loo"},
##' \code{"p_loo"} and \code{"looic"}. 
##'
##' @return A data frame with one row per observed age-specific mortality, incidence, prevalence and/or
##' remission age-specific data-point, containing leave-one-out cross validation statistics representing how
##' well the model would predict that observation if it were left out of the fit. 
##'
##' These are computed with the \pkg{loo} package.
##'
##' \code{loo_indiv} acts on the objects that are returned by running \code{\link{loo}} on \code{\link{disbayes}}
##' objects.  \code{\link{looi_disbayes}} acts directly on  \code{\link{disbayes}}
##' objects.  Both of those functions return a data frame with LOO contributions for each data point. 
##'
##' @export
loo_indiv <- function(x, agg=FALSE){
    varorig <- outcome <- age <- bias <- NULL
    if (attr(x, "disbayes_info")[["hier"]]) {
        dat <- loo_indiv_hier(x)
  } else { 
    dat <- as.data.frame(x$pointwise) %>%
      tibble::rownames_to_column("varorig") %>% 
      tidyr::extract(varorig, c("outcome", "age"), 
                     "^(.+)_prob\\[([[:digit:]]+),?[[:digit:]]?\\]$", 
                     convert=TRUE, remove = FALSE) %>%
      tidyr::extract(varorig, "bias",  
                     "^.+_prob\\[[[:digit:]]+,([[:digit:]])\\]$", 
                     convert=TRUE) %>%
      relocate(outcome, age, bias)
  }
  if (length(unique(na.omit(dat$bias)))==1) dat$bias <- NULL
  if (agg) {
    dat <- dat  %>% 
      group_by(outcome) %>% 
      summarise_at(c("elpd_loo","p_loo","looic"), sum)
  }
  dat
}

loo_indiv_hier <- function(x){
    varorig <- outcome <- age <- area <- gender <- NULL
    index_re <- paste(rep("([[:digit:]]+)",3),collapse=",")
    as.data.frame(x$pointwise) %>%
        tibble::rownames_to_column("varorig") %>% 
        tidyr::extract(varorig, c("outcome", "age", "area", "gender"), 
                       sprintf("^(.+)_prob\\[%s\\]$", index_re), 
                       convert=TRUE, remove = TRUE)
}

##' @describeIn loo_indiv Observation-level leave-one-out cross validation statistics for a disbayes model
##' @export
looi_disbayes <- function(x, agg=FALSE){
    loo_indiv(loo(x), agg=agg)
}

source("data.r")

## Basic LOO - doesn't work so well for this kind of model
## See k diagnostics

test_that("loo, standard ", {
  db1 <- disbayes(dat = ihdbristol,
                  inc_num = "inc_num", inc_denom = "inc_denom",
                  prev_num = "prev_num", prev_denom = "prev_denom",
                  mort_num = "mort_num", mort_denom = "mort_denom",
                  cf_model = "smooth",
                  eqage = 30)
  
  db2 <- disbayes(dat = ihdbristol,
                  inc_num = "inc_num", inc_denom = "inc_denom",
                  prev_num = "prev_num", prev_denom = "prev_denom",
                  mort_num = "mort_num", mort_denom = "mort_denom",
                  cf_model = "const",
                  eqage = 30)
  
  suppressWarnings(loo1 <- loo(db1))
  suppressWarnings(loo2 <- loo(db2))
  lc <- loo::loo_compare(loo1,loo2)
  if ("model" %in% colnames(lc)) {
    expect_true(
      lc[lc$model == "model1", "elpd_diff"] > lc[lc$model == "model2", "elpd_diff"]
    )
  } else {
    expect_true(lc["model1","elpd_diff"] > lc["model2","elpd_diff"])
  }
  
  ## Individual contributions  
  li <- loo_indiv(loo1)
  expect_s3_class(li, "data.frame")
  expect_equal(sum(li$elpd_loo), 
               loo1$estimates["elpd_loo","Estimate"])
  suppressWarnings(lid <- looi_disbayes(db1))
  expect_equivalent(lid, li)
  
  ## Aggregated over outcome type
  liagg <- loo_indiv(loo1, agg=TRUE)
  expect_equal(sum(liagg$elpd_loo), 
               loo1$estimates["elpd_loo","Estimate"])
})

##' Form a tidy data frame from the estimates from a disbayes fit
##'
##' Simply call this after fitting disbayes, as, e.g.
##' ```
##' res <- disbayes(...)
##' tidy(res)
##' ```
##' 
##' @importFrom generics tidy
##'
##' @param x Object returned by \code{\link{disbayes}}
##'
##' @param startyear Only used for models with time trends.  Numeric year represented by year 1 in the data. For example, set this to 1918 to convert years 1-100 to years 1918-2017.
##'
##' @param ... Other arguments (currently unused)
##'
##' @return A data frame with one row per model parameter, giving summary statistics
##' for the posterior distribution for that parameter.   For array parameters, e.g. those
##' that depend on age or area, then the age and area are returned in separate columns,
##' to make it easier to summarise and plot the results, e.g. using \pkg{ggplot2}.
##'
##' Model parameters might include, depending on the model specification, 
##'
##' * `cf`, `inc`, `rem`: Case fatality, incidence, remission rates
##'
##' * `inc_prob`, `rem_prob`, `mort_prob`, `cf_prob`:  Annual incidence, remission, mortality and case fatality risks (probabilities). 
##'
##' * `prev_prob` Prevalence (probability). 
##'
##' * `state_probs` State occupancy probabilities.
##'
##' * `beta`, `beta_inc` Coefficients of the spline basis for case fatality and incidence respectively.
##'
##' * `lambda_cf`, `lambda_inc` Smoothness parameters of the spline functions.
##'
##' * `prevzero` Prevalence at age zero
##'
##' * `cfbase` Case fatality at the baseline age (only in models where case fatality is increasing).
##'
##' * `dcf` Annual increments in case fatality (only in models where case fatality is increasing).
##'
##' * `bias_loghr` Log hazard ratio describing bias in case fatality between datasets (only in models where `bias_model` has been set).
##'
##' For models with time trends:
##'
##' * `cf_yr`, `inc_yr`, `state_probs_yr` Case fatality rates, incidence rates and state occupancy probabilities in years prior to the current year.  `cf` and `inc` refer to the rates for the current year, the one represented in the data.
##'
##'
##' Only for hierarchical models:
##'
##' * `mean_inter`, `mean_slope`,`sd_inter`,`sd_slope`.  Mean and standard deviation for random effects distribution for the intercept and slope of log case fatality.
##'
##' * `lambda_cf_male`, `lambda_inc_male`.  Smoothness of the additive gender effect on case fatality and incidence.
##'
##' * `bareat` Area-level contribution to spline basis coefficients.
##'
##' * `barea`  Normalised spline basis coefficients.
##'
##' @md
##' @export
tidy.disbayes <- function(x, startyear = 1, ...) {
  varlist <- if (x$trend) .disbayes_trend_vars else .disbayes_vars
  if (x$method %in% c("mcmc","vb"))
    res <- tidy_disbayes_full(x$fit, varlist, x$method)
  else if (x$method=="opt")
    res <- tidy_disbayes_opt(x$fit, varlist)
  if (x$trend) 
    res$year <- res$year + startyear - 1
  res
}

##' @describeIn tidy.disbayes Tidy method for hierarchical disbayes models
##' @export
tidy.disbayes_hier <- function(x, ...) {
  levs <- x[c("groups","genders")]
  if (x$method %in% c("mcmc","vb"))
    tidy_disbayes_full(x$fit, varlist=.disbayes_hier_vars, x$method, levs)
  else if (x$method=="opt")
    tidy_disbayes_opt(x$fit, varlist=.disbayes_hier_vars, levs)
}

get_opt_quantiles <- function(opt){
  has_sample <- (!is.null(opt$theta_tilde) && (nrow(opt$theta_tilde)>1))
  if (has_sample){
    quantiles <- as.data.frame(
      matrixStats::colQuantiles(opt$theta_tilde, probs=c(0.025, 0.25, 0.5, 0.75, 0.975), na.rm=TRUE)
    )
  }
  else quantiles <- NULL
}

tidy_disbayes_opt <- function(opt, varlist, levs=NULL){
  varorig <- NULL
  ests <- data.frame(varorig = names(opt$par), mode=opt$par, row.names=NULL) %>% 
    tidyr::extract(varorig, "var", "(.+)\\[.+\\]", remove=FALSE)
  stats <- "mode"
  quantiles <- get_opt_quantiles(opt)
  if (!is.null(quantiles)){
      ests <- cbind(ests, quantiles)
      stats <- c(stats, names(quantiles))   
  }
  tidy_stansumm(ests, varlist, stats, levs)
}

tidy_disbayes_full <- function(fit, varlist, method, levs=NULL, ...) {
    varorig <- NULL
    summ <- rstan::summary(fit)$summary %>% 
                              as.data.frame() %>%
                              rownames_to_column("varorig") %>%
                              tidyr::extract(varorig, "var", "(.+)\\[.+\\]", remove=FALSE)
    stats_ests <- c("mean", "se_mean", "sd", "2.5%", "25%", "50%", "75%", "97.5%", "n_eff")
    stats_diag <- if (method=="mcmc") "Rhat" else "khat"
    stats <- c(stats_ests, stats_diag)
    tidy_stansumm(summ, varlist, stats, levs)
}

.disbayes_vars <- list(
    age = list(indnames = "age",
               varnames = c("cf","inc_par","cf_par","rem_par",
                            "dcf","rem","rem_prob","mort_prob","cf_prob")), 
    agebias =  list(indnames = c("age", "bias"),
                    varnames = c("inc", "inc_prob", "prev_prob")),
    agebiasstate = list(indnames = c("age", "bias", "state"),
                        varnames = "state_probs"),
    term = list(indnames = "term", 
                varnames = c("beta", "beta_inc")),
    const = c("lambda_cf","lambda_inc","prevzero","cfbase","bias_loghr"),
    redundant = c("inc_par","cf_par","rem_par")
)
attr(.disbayes_vars, "numerics") <- c("age", "bias", "state", "term")
attr(.disbayes_vars, "order") <- c("age", "bias", "state", "term")

.disbayes_trend_vars <- list(
    age = list(indnames="age",
               varnames = c("cf", "inc_par", "rem_par", "rem",
                            "rem_prob", "mort_prob","cf_prob")),
    ageyear = list(indnames = c("age", "year"),
                   varnames = c("cf_yr")),
    agebias = list(indnames = c("age", "bias"),
                   varnames = c("inc","inc_prob","prev_prob")),
    ageyearbias = list(indnames = c("age", "year", "bias"),
                       varnames = c("inc_yr")),
    ageyearbiasstate = list(indnames = c("age","year","bias","state"),
                            varnames = c("state_probs_yr")),
    term = list(indnames = "term", 
                varnames = c("beta", "beta_inc")),
    const = c("lambda_cf","lambda_inc","prevzero","cfbase","bias_loghr"),
    redundant = c("inc_par","cf_par","rem_par")
)
attr(.disbayes_trend_vars, "numerics") <- c("age","year","bias", "state", "term")
attr(.disbayes_trend_vars, "order") <- c("age", "year", "bias", "state", "term")

.disbayes_hier_vars <- list(
    ageareagender = list(
        indnames = c("age","area","gender"), 
        varnames =  c("inc","cf","dcf","inc_prob","prev_prob",
                      "mort_prob","cf_prob","rem","rem_prob")
    ),
    ageareagenderstate = list(
        indnames = c("age","area","gender","state"), 
        varnames =  c("state_probs")
    ),
    agegender = list(
        indnames = c("age", "gender"), 
        varnames = c("rem_par")
    ), 
    termarea = list(indnames = c("term", "area"),
                    varnames = c("barea","bareat")),
    termareagender = list(
        indnames = c("term", "area", "gender"),
        varnames = c("beta","beta_inc")),
    areagender = list(
        indnames = c("area", "gender"), 
        varnames = c("prevzero")),
    area = list(
        indnames = c("area"),
        varnames = c("lcfbase")),
    term = list(indnames="term",
                varnames = c("bmale", "sd_inter", "mean_slope", "sd_slope",
                             "lambda_cf","lambda_cf_male","lambda_inc")),
    const = c("mean_inter"),
    redundant = c("inc_par","cf_par","rem_par")
)
attr(.disbayes_hier_vars, "numerics") <- c("age", "state", "term")
attr(.disbayes_hier_vars, "factors") <- data.frame(vars = c("area","gender"),
                                                   levs = c("groups", "genders"),
                                                   stringsAsFactors = FALSE)
attr(.disbayes_hier_vars, "order") <- c("age", "gender", "area", "state", "term")


## Convert Stan summary output to a tidy data frame with indices extracted as different variables
## I am reinventing the wheel somewhat here
## tidybayes and ggmcmc have similar functionality, but work on the draws rather than the output of rstan::summary. 
## tidybayes::spread_draws has one col per variable and rows for different indices, iterations and draws
## gather_draws does the same in long format with a .variable col
## Then ggdist::median_qi is used to get summary statistics

tidy_stansumm <- function(summ, varlist, stats, levs=NULL){
    var <- varorig <- NULL
    summc <- summ %>%
      filter(var %in% varlist$const) %>%
      select(-varorig)

    varnc <- varlist[!(names(varlist) %in% c("const","redundant"))]
    nvartypes <- length(varnc)
    summs <- vector(nvartypes, mode="list")
    for (i in seq_along(varnc)){
        indnames <- varnc[[i]]$indnames 
        ninds <- length(indnames)
        pattern <- paste0(".+\\[", paste(rep("([[:digit:]]+)",ninds), collapse=","), "\\]")
        summs[[i]] <- summ %>%
            filter(var %in% varnc[[i]]$varnames) %>%
            tidyr::extract(varorig, varnc[[i]]$indnames, pattern)
    }

    summ <- summc
    for (i in seq_along(varnc)){
        joinvars <- c("var", stats,
                      intersect(names(summ), varnc[[i]]$indnames))
        summ <- summ %>% full_join(summs[[i]], by=joinvars)
    }
    for (i in attr(varlist, "numerics")){
        if (i %in% names(summ))
            summ[[i]] <- as.numeric(summ[[i]])
    }
    summ <- summ %>% relocate(all_of(c("var", attr(varlist, "order"))))
    facs <- attr(varlist, "factors")
    if (!is.null(facs)){
      for (i in 1:nrow(facs)){
        if (!is.null(levs[[facs$levs[i]]]))
          summ[[facs$vars[i]]] <- factor(summ[[facs$vars[i]]], labels=levs[[facs$levs[i]]])
        else summ[[facs$vars[i]]] <- NULL
      }
    }
    summ <- summ[!(summ$var %in% varlist$redundant),]
    summ$age <- summ$age - 1
    summ
}

##' Extract observed data from a disbayes model fit
##'
##' @param x Fitted \code{\link{disbayes}} model 
##'
##' @return A data frame with columns \code{num} and \code{denom} giving the incidence, prevalence and mortality (and remission if used) numerators and denominators used in the model fit.  The column \code{var} indicates which of incidence, prevalence etc. the numbers refer to.   The column \code{prob} is derived as \code{num} divided by \code{denom}.  Columns \code{lower} and \code{upper} define credible intervals for the "data-based" point estimate \code{prob}, obtained from the Beta posterior assuming a Beta(0.5, 0.5) prior.
##'
##' This "data-based" point estimate can be compared with estimates from the model using the functions \code{\link{plotfit_data_disbayes}} and \code{\link{plotfit_disbayes}}. 
##' 
##' @export 
tidy_obsdat <- function(x){
    if (inherits(x, "disbayes_hier")) return(tidy_obsdat_hier(x))
    dat <- x$dat
    inc <- data.frame(var="inc", num = dat$inc_num, denom = dat$inc_denom)
    prev <- data.frame(var="prev", num = dat$prev_num, denom = dat$prev_denom)
    mort <- data.frame(var="mort", num = dat$mort_num, denom = dat$mort_denom)
    tdat <- rbind(inc, prev, mort)
    if (dat$remission) {
        rem <- data.frame(var="rem", num = dat$rem_num, denom = dat$rem_denom)
        tdat <- rbind(tdat, rem)
    }
    tdat$age <- rep(1:dat$nage, length.out = nrow(tdat)) - 1
    tdat$prob <- tdat$num/tdat$denom
    tdat$lower <- qbeta(0.025, tdat$num+0.5, tdat$denom-tdat$num+0.5)
    tdat$upper <- qbeta(0.975, tdat$num+0.5, tdat$denom-tdat$num+0.5)
    tdat
}

tidy_obsdat_hier <- function(x){
  dat <- x$stan_data
  dims <- dim(dat$mort_num)
  vars <- c("inc","prev","mort")
  if (dat$remission) vars <- c(vars, "rem")
  res <- vector(length(vars), mode="list")
  for (i in seq_along(vars)){
    tdat <- array_indvecs(dims)
    names(tdat) <- c("age", "area", "gender")
    tdat$age <- tdat$age - 1
    tdat$num <- as.vector(dat[[sprintf("%s_num",vars[i])]])
    tdat$denom <- as.vector(dat[[sprintf("%s_denom",vars[i])]])
    tdat$var <- vars[i]
    tdat$prob <- tdat$num/tdat$denom
    tdat$lower <- qbeta(0.025, tdat$num+0.5, tdat$denom-tdat$num+0.5)
    tdat$upper <- qbeta(0.975, tdat$num+0.5, tdat$denom-tdat$num+0.5)
    res[[i]] <- tdat
  }
  do.call("rbind", res)
}

##' Create tidy data for a check of observed against fitted outcome probability estimates
##' from disbayes
##'
##' @param x Fitted model from \code{\link{disbayes}}
##'
##' @return A data frame containing observed data in the form of outcome probabilities, as extracted by \code{\link{tidy_obsdat}}, and estimates of the corresponding probability parameters from the fitted model. 
##'
##' @export
plotfit_data_disbayes <- function(x){
    var <- age <- prob <- lower <- upper <- source <- mode <- `50%` <- `2.5%` <- `97.5%` <- NULL
    datobs <- tidy_obsdat(x) %>% 
        dplyr::select(var, age, prob, lower, upper) %>% 
        dplyr::mutate(source="Observed")
    vars <- c("inc_prob","prev_prob","mort_prob")
    if (x$dat$remission) vars <- c(vars, "rem_prob")
    res <- tidy(x)
    res$prob <- if(is.null(res$mode)) res$`50%` else res$mode
    res <- res %>% 
        dplyr::filter(var %in% vars) %>%
        dplyr::rename(lower=`2.5%`, upper=`97.5%`) %>%
        dplyr::select(var, age, prob, lower, upper) %>% 
        dplyr::mutate(source="Fitted",
                      var = gsub("_prob","",var)) %>%
        dplyr::full_join(datobs, by = c("var", "age", "prob", "lower", "upper", "source"))
    res$var[res$var=="inc"] <- "Incidence"
    res$var[res$var=="prev"] <- "Prevalence"
    res$var[res$var=="mort"] <- "Mortality"
    if (any(res$var=="rem")) res$var[res$var=="rem"] <- "Remission"
    res
}

##' Graphical check of observed against fitted outcome probabilities from disbayes
##'
##' The data behind the plot can be produced using \code{\link{plotfit_data_disbayes}},
##' to enable customised plots to be produced by hand with \code{ggplot2}.
##'
##' @inheritParams plotfit_data_disbayes
##'
##' @param agemin Minimum age to show on the horizontal axis.
##'
##' @return A \code{ggplot2} object containing the plot.
##'
##' @export
plotfit_disbayes <- function(x, agemin=50){
    age <- prob <- lower <- upper <- source <- NULL
    res <- plotfit_data_disbayes(x) %>% 
        dplyr::filter(age>agemin)
    ggplot2::ggplot(res, ggplot2::aes(x=age, y=prob, col=source)) + 
        ggplot2::geom_ribbon(ggplot2::aes(ymin=lower, ymax=upper, fill=source),
                             alpha=0.1) +
        ggplot2::geom_line() + 
        ggplot2::facet_wrap(~var, nrow=1, scales="free_y") + 
        ggplot2::ylab("")
}

## Extract samples from optimisation output
## Only implemented for parameters with three indices

opt_extract_nonhier <- function(xfit, par){
  varorig <- NULL
  sam <- xfit$theta_tilde
  parnames <- colnames(sam)
  sam <- sam[,grep(sprintf("^%s\\[.+", par), parnames)]
  parnames <- colnames(sam)
  indvars <- if (par=="mort_prob") "age" else c("age","bias")
  ninds <- length(indvars)
  pattern <- paste0(par, "\\[", paste(rep("([[:digit:]]+)",ninds), collapse=","), "\\]")
  dat <- data.frame(varorig = parnames)
  dat <- dat %>%
    tidyr::extract(varorig, indvars, pattern, convert=TRUE)
  dat$age <- dat$age - 1
  nsam <- nrow(sam)
  long_inds <- rep(1:nrow(dat), each=nsam)
  dat_long <- dat[long_inds,,drop=FALSE]
  dat_long$var <- as.vector(sam)
  dat_long$sam <- 1:nsam
  rownames(dat_long) <- NULL
  dat_long
}

## Not used currently

opt_extract_hier <- function(xfit, par){
  varorig <- NULL
  sam <- xfit$theta_tilde
  parnames <- colnames(sam)
  sam <- sam[,grep(sprintf("^%s\\[.+", par), parnames)]
  parnames <- colnames(sam)
  ninds <- 3
  pattern <- paste0(par, "\\[", paste(rep("([[:digit:]]+)",ninds), collapse=","), "\\]")
  dat <- data.frame(varorig = parnames)
  dat <- dat %>%
    tidyr::extract(varorig, c("age","area","gender"), pattern, convert=TRUE)
  dat$age <- dat$age - 1
  nsam <- nrow(sam)
  long_inds <- rep(1:nrow(dat), each=nsam)
  dat_long <- dat[long_inds,,drop=FALSE]
  dat_long$var <- as.vector(sam)
  rownames(dat_long) <- NULL
  dat_long
}

##' Conflict p-values
##'
##' A test of the hypothesis that the direct data on a disease outcome give the same 
##' information about that outcome as an indirect evidence synthesis obtained from a fitted \code{\link{disbayes}}
##' model.   The outcome may be annual incidence, mortality, remission probabilities, 
##' or prevalence. 
##' 
##' Hierarchical models are not supported in this function.  
##' 
##' @param x A fitted \code{\link{disbayes}} model.
##' 
##' @param varname Either \code{inc}, \code{prev}, \code{mort} or \code{rem}.
##' 
##' @return A data frame with columns indicating age, gender and area. 
##' 
##' \code{p1} is a "one-sided" p-value for the null hypothesis that \eqn{r_{obs}=r_{fit}} against 
##' the alternative that \eqn{r_{obs} > r_{fit}},
##' 
##' \code{p2} is the two-sided p-value for the null hypothesis that \eqn{r_{obs}=r_{fit}} against 
##' the alternative that \eqn{r_{obs}} is not equal to \eqn{r_{fit}},
##' 
##' where \eqn{r_{obs}} is the rate informed only by direct data, and \eqn{r_{fit}} is the rate
##' informed by evidence synthesis.  Therefore if the evidence synthesis excludes the 
##' direct data, then these are interpreted as "conflict" p-values (see Presanis et al. 2013). 
##' 
##' In each case, a small p-value favours the alternative hypothesis.
##'
##' @references Presanis, A. M., Ohlssen, D., Spiegelhalter, D. J. and De Angelis, D. (2013) 
##' Conflict diagnostics in directed acyclic graphs, with applications in Bayesian evidence 
##' synthesis. Statistical Science, 28, 376-397.
##' 
##' @export
conflict_disbayes <- function(x, varname){
  ## Extract observed and fitted value of "varname_prob" by sample, age [ area and gender ]
  ## as tidy data frame 
    datobs <- tidy_obsdat(x)
    datobs <- datobs[datobs$var==varname,]
    if (inherits(x$fit, "stanfit")){
      fitted <- rstan::extract(x$fit, pars=paste(varname, "prob", sep="_"))[[1]]
      nsam <- dim(fitted)[1]
      nage <- dim(fitted)[2]
      if (inherits(x, "disbayes_hier")){
        narea <- dim(fitted)[3]
        ngender <- dim(fitted)[4]
      } else {
        narea <- ngender <- 1
        datobs$area <- datobs$gender <- 1
        fitted <- array(as.vector(fitted), dim=c(dim(fitted), 1, 1))
      }
      res <- datobs[,c("age","area","gender")]
      fitted_long <- res[rep(1:nrow(res), each=nsam),]
      fitted_long$sam <- 1:nsam # auto replicated
      fitted_long$var <- as.vector(fitted)
    } else {
      fitted_long <- opt_extract_nonhier(x$fit, paste0(varname,"_prob"))
      if (is.null(fitted_long$area)) fitted_long$area <- 1
      if (is.null(fitted_long$gender)) fitted_long$gender <- 1
      nsam <- length(unique(fitted_long$sam))
      nage <- length(unique(fitted_long$age))
      narea <- length(unique(fitted_long$area))
      ngender <- length(unique(fitted_long$gender))
      res <- fitted_long[fitted_long$sam==1,c("age","area","gender")]
    }
    res$p1 <- res$p2 <- NA 
    
    ages <- 0:(nage-1)
    for (a in 1:nage){
      for (j in 1:narea){
        for (g in 1:ngender){
          ind <- (res$age==ages[a]) & (res$area==j) & (res$gender==g)
          num <- datobs$num[ind]
          denom <- datobs$denom[ind]
          obssam <- rbeta(nsam, num + 0.5, denom - num + 0.5)
          fitsam <- fitted_long$var[ind]
          res$p1[ind] <- mean(obssam < fitsam)
          res$p2[ind] <- 2*min(res$p1[ind], 1 - res$p1[ind])
        }
      }
    }
    res
}

source("data.r")

test_that("standard disbayes model, optimisation",{
  set.seed(1)
  db <- disbayes(dat = ihdbristol,
                 inc_num = "inc_num", inc_denom = "inc_denom",
                 prev_num = "prev_num", prev_denom = "prev_denom",
                 mort_num = "mort_num", mort_denom = "mort_denom",
                 eqage = 40, seed=1)
  res <- tidy(db) %>% filter(var=="cf")
  expect_equal(res$mode[60], 0.0172128687091707, tol=1e-03)
  expect_equal(res$`50%`[60], 0.0173084702459764, tol=1e-03)
  
  od <- tidy_obsdat(db)
  expect_equal(od$denom[1], ihdbristol$inc_denom[1])
  if (interactive()){
    plotfit_data_disbayes(db)
    plotfit_disbayes(db)
  }
  
  cd <- conflict_disbayes(db, varname="inc")
  expect_lte(cd$p1[1], 1)
  expect_gte(cd$p1[1], 0)
})

source("data.r")

test_that("fit and conflict checks",{
  set.seed(1)
  db <- disbayes(dat = ihdbristol,
                    inc_num = "inc_num", inc_denom = "inc_denom",
                    prev_num = "prev_num", prev_denom = "prev_denom",
                    mort_num = "mort_num", mort_denom = "mort_denom",
                    eqage = 40, method="mcmc", algorithm="Fixed_param", 
                    chains=1, iter=100)
  expect_s3_class(db, "disbayes")
  
  od <- tidy_obsdat(db)
  expect_equal(od$denom[1], ihdbristol$inc_denom[1])
  if (interactive()){
    plotfit_data_disbayes(db)
    plotfit_disbayes(db)
  }
  
  cd <- conflict_disbayes(db, varname="mort")
  expect_lte(cd$p1[1], 1)
  expect_gte(cd$p1[1], 0)
})

library(dplyr)
library(tidyr)

ihdbristol <- ihdengland %>% 
  dplyr::filter(area=="Bristol", gender=="Male") %>%
  mutate(
    mort_prob = qbeta(0.5, mort_num+0.5, mort_denom-mort_num+0.5),
    mort_lower = qbeta(0.025, mort_num+0.5, mort_denom-mort_num+0.5),
    mort_upper = qbeta(0.975, mort_num+0.5, mort_denom-mort_num+0.5),
    inc_prob = qbeta(0.5, inc_num+0.5, inc_denom-inc_num+0.5),
    inc_lower = qbeta(0.025, inc_num+0.5, inc_denom-inc_num+0.5),
    inc_upper = qbeta(0.975, inc_num+0.5, inc_denom-inc_num+0.5),
    prev_prob = qbeta(0.5, prev_num+0.5, prev_denom-prev_num+0.5),
    prev_lower = qbeta(0.025, prev_num+0.5, prev_denom-prev_num+0.5),
    prev_upper = qbeta(0.975, prev_num+0.5, prev_denom-prev_num+0.5),
    rem_num = 10,
    rem_denom = 100)

ihdmale <- ihdengland %>% 
  dplyr::filter(gender=="Male")

## mdat <- remission, eqage, const_rem 
## min <- cf_init

init_rates <- function(dat, mdata, idata,...){
    inc_init <- init_rate("inc", dat)
    rem_init <- init_rate("rem", dat, agg=mdata$const_rem)
    optu <- fit_unsmoothed_opt(dat, inc_init=inc_init, rem_init=rem_init,
                               mdata=mdata, idata=idata)
    optdf <- tidy_disbayes_opt(optu, .disbayes_vars)
    cf <- optdf$mode[optdf$var=="cf"] 
    inc <- optdf$mode[optdf$var=="inc"]
    rem <- optdf$mode[optdf$var=="rem"]
    eqage <- mdata$eqage
    for (i in 1:eqage){
        cf[i] <- cf[eqage+1]
        inc[i] <- inc[eqage+1]
        rem[i] <- rem[eqage+1]
    }
    init_eqage_hi <- 90 
    for (i in init_eqage_hi:dat$nage){
        cf[i] <- cf[init_eqage_hi-1]
        inc[i] <- inc[init_eqage_hi-1]
        rem[i] <- rem[init_eqage_hi-1]
    }
    if (mdata$const_rem) rem=rem_init
    list(cf=cf, inc=inc, rem=rem, optu=optu)
}

init_rate <- function(rate, dat, agg=FALSE){
    default <- 0.001
    nums <- dat[[sprintf("%s_num",rate)]]
    denoms <- dat[[sprintf("%s_denom",rate)]]
    if (agg) { nums <- sum(nums); denoms <- sum(denoms) }
    rcrude <- nums / denoms 
    rcrude[is.na(rcrude)] <- default
    rcrude[is.nan(rcrude)] <- default
    rcrude <- pmax(default, rcrude)
    rcrude 
}

fit_unsmoothed_opt <- function(dat, inc_init=NULL, rem_init=NULL,
                               mdata, idata){
    if (is.null(inc_init)) inc_init <- init_rate("inc", dat)
    if (is.null(rem_init)) rem_init <- init_rate("rem", dat, mdata$const_rem)
    nage <- dat$nage
    Xdummy <- matrix(0, nrow=nage, ncol=2)
    datstanu <- c(dat, mdata)
    ## quantities in the data that are different for the purpose of this training model
    data_fixed <- list(smooth_cf =  0, smooth_inc = 0, smooth_rem = 0, 
                       const_cf = 0, trend = 0, nyr=1, nbias=1,
                       bias = 1, incdata_ind = 1, prevdata_ind = 1,
                       increasing_cf=0, K=2, X=Xdummy, 
                       inc_trend = array(1, dim=c(nage,1)),
                       cf_trend = array(1, dim=c(nage,1)),
                       scf_isfixed=0, sinc_isfixed=0, srem_isfixed=0,
                       lambda_cf_fixed=0, lambda_inc_fixed=0, lambda_rem_fixed=0)
    for (i in names(data_fixed))
        datstanu[[i]] <- data_fixed[[i]]
    initu <- list(cf_par = rep(idata$cf_init,nage), 
                  rem_par = if (mdata$remission) as.array(rem_init) else numeric(),
                  inc_par = inc_init, 
                  prevzero = if (mdata$prev_zero) as.array(max(dat$prev_num[1],1)/max(dat$prev_denom[1],2)) else numeric())
    opt <- rstan::optimizing(stanmodels$disbayes, data = datstanu, 
                             init = initu, 
                             hessian = FALSE)
    class(opt) <- "disopt"
    opt
}

## Obtains spline basis terms, and initial values for their coefficients given
## estimates of incidence or CF from unsmoothed model.

init_smooth <- function(y, eqage, eqagehi, s_opts=NULL){
    x <- NULL # define unbound variables to satisfy R CMD check 
    nage <- length(y)
    age <- agecons <- 1:nage 
    agecons[1:eqage] <- eqage
    if (!is.null(eqagehi) && eqagehi < nage)
        agecons[eqagehi:nage] <- eqagehi
    sm <- mgcv::smoothCon(s(x),
                          data=data.frame(x=agecons),
                          diagonal.penalty=TRUE)[[1]]
    X <- sm$X
    beta <- coef(lm(y ~ X - 1))
    list(X=X, beta=beta)
}


## Form constant initial value list to supply to Stan 

initlist_const <- function(initrates, cf_smooth, inc_smooth, remission, rem_smooth, 
                           eqage, smooth_inc, smooth_cf, const_cf,
                           increasing_cf, smooth_rem, const_rem, nbias,
                           scf_isfixed, sinc_isfixed, srem_isfixed){
    lam_init <- laminc_init <- lamrem_init <- 0.5 
    beta_init <- cf_smooth$beta 
    betainc_init <- inc_smooth$beta
    betarem_init <- rem_smooth$beta
    list(inc_par = if (smooth_inc) numeric() else initrates$inc,
         rem_par = if (remission && !smooth_rem) as.array(initrates$rem) else numeric(),
         beta = if (smooth_cf && !const_cf) beta_init else numeric(),
         lambda_cf = if (smooth_cf && !scf_isfixed) as.array(lam_init) else numeric(),
         beta_inc = if (smooth_inc) betainc_init else numeric(),
         lambda_inc = if (smooth_inc && !sinc_isfixed) as.array(laminc_init) else numeric(),
         beta_rem = if (smooth_rem) betarem_init else numeric(),
         lambda_rem = if (smooth_rem && !srem_isfixed) as.array(lamrem_init) else numeric(),
         cfbase = if (const_cf | increasing_cf) as.array(initrates$cf[eqage]) else numeric(),
         bias_loghr = if (nbias > 1) as.array(rnorm(1)) else numeric()
         )
}

## Form initial value list random generating function to supply to Stan 

initlist_random <- function(nage, initrates, cf_smooth, inc_smooth, remission, rem_smooth,
                            eqage, smooth_inc, smooth_cf, const_cf, increasing_cf,
                            smooth_rem, const_rem, nbias, scf_isfixed, sinc_isfixed, srem_isfixed){
    lam_init <- laminc_init <- lamrem_init <- 0.5
    beta_init <- cf_smooth$beta 
    betainc_init <- inc_smooth$beta
    betarem_init <- rem_smooth$beta
    inits <- function(){
        list(
            inc_par = rlnorm(nage*(1-smooth_inc),
                             meanlog=log(initrates$inc),
                             sdlog=initrates$inc/10),
            rem_par = as.array(rlnorm(remission*(1-smooth_rem)*(nage*(1 - const_rem) + 1*const_rem),
                                      meanlog=log(initrates$rem),
                                      sdlog=initrates$rem/10)),
            beta = if (smooth_cf) rnorm(length(beta_init)*(1 - const_cf),
                                        mean=beta_init,
                                        sd=abs(beta_init)/10) else numeric(),
            lambda_cf = as.array(rlnorm(length(lam_init)*smooth_cf*(1-scf_isfixed),
                                     meanlog=log(lam_init),
                                     sdlog=lam_init/10)),
            beta_inc = if (smooth_inc) rnorm(length(betainc_init),
                                             mean=betainc_init,
                                             sd=abs(betainc_init)/10) else numeric(),
            lambda_inc = as.array(rlnorm(length(laminc_init)*smooth_inc*(1-sinc_isfixed),
                                         meanlog=log(laminc_init),
                                         sdlog=laminc_init/10)),
            beta_rem = if (remission && !const_rem) rnorm(length(betarem_init)*smooth_rem,
                                            mean=betarem_init, sd=abs(betarem_init)/10) else numeric(),
            lambda_rem = as.array(rlnorm(length(lamrem_init)*smooth_rem*(1-srem_isfixed),
                                         meanlog=log(lamrem_init),
                                         sdlog=lamrem_init/10)),
            cfbase = if (const_cf | increasing_cf) as.array(initrates$cf[eqage]) else numeric(),
            bias_loghr = if (nbias > 1) as.array(rnorm(1)) else numeric()
        )
    }
    inits
}

fit_unsmoothed <- function(dat, inc_init=NULL, rem_init=NULL, 
                           mdata, idata, 
                           method = "mcmc",
                           iter = 1000, 
                           stan_control = NULL, ...){
    if (is.null(inc_init)) inc_init <- init_rate("inc", dat)
    if (is.null(rem_init)) rem_init <- init_rate("rem", dat)
    nage <- dat$nage
    Xdummy <- matrix(0, nrow=nage, ncol=2)
    datstanu <- c(dat, mdata)
    ## quantities in the data that are different for the purpose of the unsmoothed model
    ## everything else is passed from dat (observed data) or mdata (model spec)
    data_fixed <- list(smooth_cf =  0, smooth_inc = 0, const_cf = 0, trend = 0, nyr=1, nbias=1,
                       bias = 1, incdata_ind = 1, prevdata_ind = 1,
                       increasing_cf=0, K=2, X=Xdummy, 
                       inc_trend = array(1, dim=c(nage,1)),
                       cf_trend = array(1, dim=c(nage,1)),
                       scf_isfixed=0, sinc_isfixed=0, srem_isfixed=0,
                       lambda_cf_fixed=0, lambda_inc_fixed=0, lambda_rem_fixed=0)
    for (i in names(data_fixed))
        datstanu[[i]] <- data_fixed[[i]]
    initu <- function(){
        list(cf_par = rnorm(nage, mean=idata$cf_init, sd=idata$cf_init/10),
             rem_par = rnorm(mdata$remission*(1 - mdata$smooth_rem)*(nage*(1 - mdata$const_rem) + 1*mdata$const_rem),
                             mean=rem_init, sd=rem_init/10),
             inc_par = rnorm(nage, mean=inc_init, sd=inc_init/10))
    }
    if (method=="mcmc") { 
        fitu <- rstan::sampling(stanmodels$disbayes, data = datstanu, 
                                init = initu, include = FALSE, pars=c("beta","lambda"),
                                iter = iter, 
                                control = stan_control, ...)
    } else { 
        fitu <- rstan::vb(stanmodels$disbayes, data = datstanu, 
                          init = initu, include = FALSE, pars=c("beta","lambda"))
    }
    fitu 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.R
\name{tidy_obsdat}
\alias{tidy_obsdat}
\title{Extract observed data from a disbayes model fit}
\usage{
tidy_obsdat(x)
}
\arguments{
\item{x}{Fitted \code{\link{disbayes}} model}
}
\value{
A data frame with columns \code{num} and \code{denom} giving the incidence, prevalence and mortality (and remission if used) numerators and denominators used in the model fit.  The column \code{var} indicates which of incidence, prevalence etc. the numbers refer to.   The column \code{prob} is derived as \code{num} divided by \code{denom}.  Columns \code{lower} and \code{upper} define credible intervals for the "data-based" point estimate \code{prob}, obtained from the Beta posterior assuming a Beta(0.5, 0.5) prior.

This "data-based" point estimate can be compared with estimates from the model using the functions \code{\link{plotfit_data_disbayes}} and \code{\link{plotfit_disbayes}}.
}
\description{
Extract observed data from a disbayes model fit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.R
\name{plotfit_data_disbayes}
\alias{plotfit_data_disbayes}
\title{Create tidy data for a check of observed against fitted outcome probability estimates
from disbayes}
\usage{
plotfit_data_disbayes(x)
}
\arguments{
\item{x}{Fitted model from \code{\link{disbayes}}}
}
\value{
A data frame containing observed data in the form of outcome probabilities, as extracted by \code{\link{tidy_obsdat}}, and estimates of the corresponding probability parameters from the fitted model.
}
\description{
Create tidy data for a check of observed against fitted outcome probability estimates
from disbayes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.R
\name{plotfit_disbayes}
\alias{plotfit_disbayes}
\title{Graphical check of observed against fitted outcome probabilities from disbayes}
\usage{
plotfit_disbayes(x, agemin = 50)
}
\arguments{
\item{x}{Fitted model from \code{\link{disbayes}}}

\item{agemin}{Minimum age to show on the horizontal axis.}
}
\value{
A \code{ggplot2} object containing the plot.
}
\description{
The data behind the plot can be produced using \code{\link{plotfit_data_disbayes}},
to enable customised plots to be produced by hand with \code{ggplot2}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/disbayes.R
\name{plot.disbayes}
\alias{plot.disbayes}
\title{Quick and dirty plot of estimates from disbayes models against age}
\usage{
\method{plot}{disbayes}(x, variable = "cf", ...)
}
\arguments{
\item{x}{Object returned by \code{\link{disbayes}}}

\item{variable}{Name of the variable of interest to plot against age, by default case fatality rates.}

\item{...}{Other arguments. Currently unused}
}
\value{
A \code{ggplot2} object that can be printed to show the plot, or customised by adding \code{geom}s.

Better plots can be drawn by \code{tidy}ing the object returned by \code{disbayes}, and using \code{ggplot2} directly on the tidy data frame that this produces. See the vignette for examples.
}
\description{
Posterior medians and 95% credible intervals for a quantity of interest are plotted against year of age.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/disbayes-package.R
\docType{package}
\name{disbayes-package}
\alias{disbayes-package}
\title{The 'disbayes' package.}
\description{
Bayesian evidence synthesis for chronic disease epidemiology
}
\references{
Stan Development Team (2019). RStan: the R interface to Stan. R package version 2.19.2. https://mc-stan.org
}
\seealso{
Useful links:
\itemize{
  \item \url{https://chjackson.github.io/disbayes/}
  \item Report bugs at \url{https://github.com/chjackson/disbayes/issues}
}

}
\author{
\strong{Maintainer}: Christopher Jackson \email{chris.jackson@mrc-bsu.cam.ac.uk} (\href{https://orcid.org/0000-0002-6656-8913}{ORCID}) [copyright holder]

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ci2num.R
\name{ci2num}
\alias{ci2num}
\title{Convert a proportion and credible interval to a numerator and denominator}
\usage{
ci2num(est, lower, upper, epsilon = 0.5, denom0 = 1000)
}
\arguments{
\item{est}{Point estimate}

\item{lower}{Lower 95\% credible limit}

\item{upper}{Upper 95\% credible limit}

\item{epsilon}{If any of \code{lower} are zero, then they are replaced by the minimum of \code{epsilon} and \code{est/2}.  Similarly values of 1 for \code{upper} are replaced by the maximum of \code{1-epsilon} and \code{(1+est)/2}.}

\item{denom0}{Denominator to use as a default when the point estimate is exactly 0 or 1 (which is not compatible with the beta distribution).  Should correspond to a guess of the population size used to produce the estimate,
which should be no greater than the actual population of the area, and usually less.  Should be either a scalar, or a vector of the same length as \code{est} (though note if it is a vector, then only the elements where \code{est} is 1 or 0 get used).}
}
\value{
A data frame with elements \code{num} and \code{denom} corresponding to the supplied estimate and limits.
}
\description{
Estimate the number of events and denominator that contain roughly equivalent information to an estimate and uncertainty interval for a proportion, by interpreting the estimate and interval as a Beta posterior arising from a vague Beta(0.5,0.5) prior updated with the data consisting of that number and denominator.
}
\details{
Based on fitting a Beta distribution by least squares, using the method provided by the \pkg{SHELF} package.

Requires that the estimate and upper and lower limits are all distinct (except that \code{est=0} is allowed and handled specially for convenience, see \code{denom0}).  Vectors of estimates and limits may be supplied.
}
\examples{
est <- 3.00 / 100 
upper <- 3.52 / 100 
lower <- 2.60 / 100 
ci2num(est, lower, upper)

}
\references{
Oakley (2020). SHELF: Tools to Support the Sheffield Elicitation Framework. R package version 1.7.0. \url{https://CRAN.R-project.org/package=SHELF}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/disbayes.R
\name{disbayes}
\alias{disbayes}
\title{Bayesian estimation of chronic disease epidemiology from incomplete data}
\usage{
disbayes(
  data,
  inc_num = NULL,
  inc_denom = NULL,
  inc_prob = NULL,
  inc_lower = NULL,
  inc_upper = NULL,
  prev_num = NULL,
  prev_denom = NULL,
  prev_prob = NULL,
  prev_lower = NULL,
  prev_upper = NULL,
  mort_num = NULL,
  mort_denom = NULL,
  mort_prob = NULL,
  mort_lower = NULL,
  mort_upper = NULL,
  rem_num = NULL,
  rem_denom = NULL,
  rem_prob = NULL,
  rem_lower = NULL,
  rem_upper = NULL,
  age = "age",
  cf_model = "smooth",
  inc_model = "smooth",
  rem_model = "const",
  prev_zero = FALSE,
  inc_trend = NULL,
  cf_trend = NULL,
  cf_init = 0.01,
  eqage = 30,
  eqagehi = NULL,
  sprior = c(1, 1, 1),
  hp_fixed = NULL,
  rem_prior = c(1.1, 1),
  inc_prior = c(2, 0.1),
  cf_prior = c(2, 0.1),
  method = "opt",
  draws = 1000,
  iter = 10000,
  stan_control = NULL,
  bias_model = NULL,
  ...
)
}
\arguments{
\item{data}{Data frame containing some of the variables below.  The
variables below are provided as character strings naming columns in this
data frame.   For each disease measure available, one of the following three
combinations of variables must be specified:

(1) numerator and denominator (2) estimate and denominator (3) estimate
with lower and upper credible limits.

Mortality must be supplied, and at least one of incidence and prevalence.
If remission is assumed to be possible, then remission data should also be supplied (see below).

Estimates refer to the probability of having some event within a year, rather than rates.  Rates per year $r$ can be
converted to probabilities $p$ as $p = 1 - exp(-r)$, assuming the rate is constant
within the year.

For estimates based on registry data assumed to cover the whole
population, then the denominator will be the population size.}

\item{inc_num}{Numerator for the incidence data, assumed to represent the
observed number of new cases within a year among a population of size
\code{inc_denom}.}

\item{inc_denom}{Denominator for the incidence data.

The function \code{\link{ci2num}} can be used to convert a published
estimate and interval for a proportion to an implicit numerator and
denominator.

Note that to include extra uncertainty beyond that implied by a published
interval, the numerator and denominator could be multiplied by a constant,
for example, multiplying both the numerator and denominator by 0.5 would
give the data source half its original weight.}

\item{inc_prob}{Estimate of the incidence probability}

\item{inc_lower}{Lower credible limit for the incidence estimate}

\item{inc_upper}{Upper credible limit for the incidence estimate}

\item{prev_num}{Numerator for the estimate of prevalence, i.e.
number of people currently with a disease.}

\item{prev_denom}{Denominator for the estimate of prevalence (e.g. the size
of the survey used to obtain the prevalence estimate)}

\item{prev_prob}{Estimate of the prevalence probability}

\item{prev_lower}{Lower credible limit for the prevalence estimate}

\item{prev_upper}{Upper credible limit for the prevalence estimate}

\item{mort_num}{Numerator for the estimate of the mortality probability, i.e
number of deaths attributed to the disease under study within a year}

\item{mort_denom}{Denominator for the estimate of the mortality probability (e.g.
the population size, if the estimates were obtained from a comprehensive
register)}

\item{mort_prob}{Estimate of the mortality probability}

\item{mort_lower}{Lower credible limit for the mortality estimate}

\item{mort_upper}{Upper credible limit for the mortality estimate}

\item{rem_num}{Numerator for the estimate of the remission probability, i.e number
of people observed to recover from the disease within a year.

Remission
data should be supplied if remission is permitted in the model, either as
a numerator and denominator or as an estimate and lower credible interval.
Conversely, if no remission data are supplied, then remission is assumed
to be impossible.  These "data" may represent a prior judgement rather than
observation - lower denominators or wider credible limits represent
weaker prior information.}

\item{rem_denom}{Denominator for the estimate of the remission probability}

\item{rem_prob}{Estimate of the remission probability}

\item{rem_lower}{Lower credible limit for the remission estimate}

\item{rem_upper}{Upper credible limit for the remission estimate}

\item{age}{Variable in the data indicating the year of age.  This must
start at age zero, but can end at any age.}

\item{cf_model}{Model for how case fatality rate varies with age.

\code{"smooth"} (the default). Case fatality rate is modelled as a smooth function of age,
using a spline.

\code{"indep"} Case fatality rates are estimated independently for each year of age.  This may be
useful for determining how much information is in
the data. That is, if the posterior from this model is identical to the
prior for a certain age, then there is no information in the data alone
about case fatality at that age, indicating that some other structural
assumption (such as a smooth function of age) or external data are equired
to give more precise estimates.

\code{"increasing"} Case fatality rate is modelled as a smooth and increasing
function of age.

\code{"const"} Case fatality rate is modelled as constant with age.}

\item{inc_model}{Model for how incidence rates vary with age.

\code{"smooth"} (the default). Incidence rates are modelled as a smooth spline function of age.

\code{"indep"} Incidence rates for each year of age are estimated independently.}

\item{rem_model}{Model for how remission rates vary with age, which are typically
less well-informed by data, compared to incidence and case fatality.

\code{"const"} (the default). Constant remission rate over all ages.

\code{"smooth"} Remission rates are modelled as a smooth spline function of age.

\code{"indep"} Remission rates estimated independently over all ages.}

\item{prev_zero}{If \code{TRUE}, attempt to estimate prevalence at age zero
from the data, as part of the Bayesian model, even if the observed prevalence is zero.
Otherwise (the default) this is assumed to be zero if the count is zero, and estimated
otherwise.}

\item{inc_trend}{Matrix of constants representing trends in incidence
through calendar time by year of age.  There are \code{nage} rows and
\code{nage} columns, where \code{nage} is the number of years of age
represented in the data. The entry in the ith row and jth column
represents the ratio between the incidence \code{nage+j} years prior to
the year of the data, year, and the incidence in the year of the data, for
a person i-1 years of age. For example, if \code{nage=100} and the data
refer to the year 2017, then the first column refers to the year 1918 and
the last (100th) column refers to 2017.  The last column should be all 1,
unless the current data are supposed to be biased.

To produce this format from a long data frame, filter to the appropriate
outcome and subgroup, and use \code{\link[tidyr]{pivot_wider}}, e.g.

\code{trends <- ihdtrends %>% 
           filter(outcome=="Incidence", gender=="Female") %>%
            pivot_wider(names_from="year", values_from="p2017") %>% 
            select(-age, -gender, -outcome) %>% 
            as.matrix()}}

\item{cf_trend}{Matrix of constants representing trends in case fatality
through calendar time by year of age, in the same format as
\code{inc_trend}.}

\item{cf_init}{Initial guess at a typical case fatality value, for an
average age.}

\item{eqage}{Case fatalities (and incidence and remission rates) are assumed to be equal for
all ages below this age, inclusive, when using the smoothed model.}

\item{eqagehi}{Case fatalities (and incidence and remission rates) are assumed to be equal for
all ages above this age, inclusive, when using the smoothed model.}

\item{sprior}{Rates of the exponential prior distributions used to penalise
the coefficients of the spline model.   The default of 1 should adapt
appropriately to the data, but Higher values give stronger smoothing, or
lower values give weaker smoothing,  if required.

This can be a named vector with names \code{"inc","cf","rem"} in any
order, giving the prior smoothness rates for incidence, case fatality and
remission.  If any of these are not smoothed they can be excluded, e.g.
\code{sprior = c(cf=10, inc=1)}.

This can also be an unnamed vector of three elements, where the first
refers to the spline model for incidence, the second for case fatality,
the third for remission. If one of the rates (e.g. remission) is not being
modelled with a spline, any number can be supplied here and it is just
ignored.}

\item{hp_fixed}{A list with one named element for each hyperparameter
to be fixed.  The value should be either
\itemize{
\item a number (to fix the hyperparameter at this number)
\item \code{TRUE} (to fix the hyperparameter at the posterior mode from a training run
where it is not fixed)
}

If the element is either \code{NULL}, \code{FALSE}, or omitted from the list,
then the hyperparameter is given a prior and estimated as part of the Bayesian model.

The hyperparameters that can be fixed are
\itemize{
\item \code{scf} Smoothness parameter for the spline relating case fatality to age.
\item \code{sinc} Smoothness parameter for the spline relating incidence to age.
}

For example, to fix the case fatality smoothness to 1.2 and fix the incidence
smoothness to its posterior mode,
specify \code{hp_fixed = list(scf=1.2, sinc=TRUE)}.}

\item{rem_prior}{Vector of two elements giving the Gamma shape and rate parameters of the
prior for the remission rate, used in both \code{rem_model="const"} and \code{rem_model="indep"}.}

\item{inc_prior}{Vector of two elements giving the Gamma shape and rate parameters of the
prior for the incidence rate.  Only used if \code{inc_model="indep"}, for each age-specific rate.}

\item{cf_prior}{Vector of two elements giving the Gamma shape and rate parameters of the
prior for the case fatality rate.  Only used if \code{cf_model="const"}, or if \code{cf_model="indep"}, for each age-specific rate,
and for the rate at \code{eqage} in \code{cf_model="increasing"}.}

\item{method}{String indicating the inference method, defaulting to
\code{"opt"}.

If \code{method="mcmc"} then a sample from the posterior is drawn using Markov Chain Monte Carlo
sampling, via \pkg{rstan}'s \code{\link[rstan:stanmodel-method-sampling]{rstan::sampling()}} function.   This is the most
accurate but the slowest method.

If \code{method="opt"}, then instead of an MCMC sample from the posterior,
\code{disbayes} returns the posterior mode calculated using optimisation, via
\pkg{rstan}'s \code{\link[rstan:stanmodel-method-optimizing]{rstan::optimizing()}} function.
A sample from a normal approximation to the (real-line-transformed)
posterior distribution is drawn in order to obtain credible intervals.

If the optimisation fails to converge (non-zero return code), try increasing the
number of iterations from the default 1000, e.g. \code{disbayes(..., iter=10000, ...)}, or changing the algorithm to \code{disbayes(..., algorithm="Newton", ...)}.

If there is an error message that mentions \code{chol}, then
the computed Hessian matrix is not positive definite at the reported optimum, hence credible intervals
cannot be computed.
This can occur either because of numerical error in computation of the Hessian, or because the
reported optimum is wrong.  If you are willing to believe
the optimum and live without credible intervals, then set \code{draws=0} to skip
computation of the Hessian.   To examine the problematic Hessian, set
\code{hessian=TRUE,draws=0}, then look at the \code{$fit$hessian} component of the
\code{disbayes} return object.   If it can be inverted, do \code{sqrt(diag(solve()))} on the Hessian, and
check for \code{NaN}s, indicating the problematic parameters.
Otherwise, diagonal entries of the Hessian matrix that are very small
may indicate parameters that are poorly identified from the data, leading to computational
problems.

If \code{method="vb"}, then variational Bayes methods are used, via \pkg{rstan}'s
\code{\link[rstan:stanmodel-method-vb]{rstan::vb()}} function.  This is labelled as "experimental" by
\pkg{rstan}.  It might give a better approximation to the posterior
than \code{method="opt"}, but has not been investigated much for \code{disbayes} models.}

\item{draws}{Number of draws from the normal approximation to the posterior
when using \code{method="opt"}.}

\item{iter}{Number of iterations for MCMC sampling, or maximum number of iterations for optimization.}

\item{stan_control}{(\code{method="mcmc"} only). List of options supplied as the \code{control} argument
to \code{\link[rstan:stanmodel-method-sampling]{rstan::sampling()}} in \pkg{rstan} for the main model fit.}

\item{bias_model}{Experimental model for bias in the incidence estimates due
to conflicting information between the different data sources.  If
\code{bias_model=NULL} (the default) no bias is assumed, and all data are
assumed to be generated from the same age-specific incidences.

Otherwise there are assumed to be two alternative curves of incidence by
age (denoted 2 and 1) where curve 2 is related to curve 1 via a constant
hazard ratio that is estimated from the data, given a standard normal
prior on the log scale.  Three distinct curves would not be identifiable
from the data.

If \code{bias_model="inc"} then the incidence data is assumed to be
generated from curve 2, and the prevalence and mortality data from curve
1.

\code{bias_model="prev"} then the prevalence data is generated from curve
2, and the incidence and mortality data from curve 1.

If \code{bias_model="incprev"} then both incidence and prevalence data are
generated from curve 2, and the mortality data from curve 1.}

\item{...}{Further arguments passed to \code{\link[rstan:stanmodel-method-sampling]{rstan::sampling()}} to
control MCMC sampling, or \code{\link[rstan:stanmodel-method-optimizing]{rstan::optimizing()}} to control
optimisation, in Stan.}
}
\value{
A list including the following components

\code{call}: Function call that was used.

\code{fit}: An object containing posterior samples from the fitted model,
in the \code{stanfit} format returned by the \code{\link[rstan]{stan}}
function in the \pkg{rstan} package.

\code{method}:  Optimisation method that was chosen.

\code{nage}: Number of years of age in the data

\code{dat}: A list containing the input data in the form of numerators
and denominators.

\code{stan_data}: Full list of data supplied to Stan

\code{stan_inits}: Full list of parameter initial values supplied to Stan

\code{hp_fixed} Values of any hyperparameters that are fixed during the main model fit.

Use the \code{\link{tidy.disbayes}} method to return summary statistics
from the fitted models, simply by calling \code{tidy()} on the fitted model.
}
\description{
Estimates a three-state disease model from incomplete data.
It is designed to estimate case fatality and incidence, given
data on mortality and at least one of incidence and prevalence.
Remission may also be included in the data and modelled.
}
\references{
Jackson C, Zapata-Diomedi B, Woodcock J. (2023)
"Bayesian multistate modelling of incomplete chronic disease burden data"
Journal of the Royal Statistical Society, Series A, 186(1), 1-19
\doi{10.1093/jrsssa/qnac015}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.R
\name{conflict_disbayes}
\alias{conflict_disbayes}
\title{Conflict p-values}
\usage{
conflict_disbayes(x, varname)
}
\arguments{
\item{x}{A fitted \code{\link{disbayes}} model.}

\item{varname}{Either \code{inc}, \code{prev}, \code{mort} or \code{rem}.}
}
\value{
A data frame with columns indicating age, gender and area. 

\code{p1} is a "one-sided" p-value for the null hypothesis that \eqn{r_{obs}=r_{fit}} against 
the alternative that \eqn{r_{obs} > r_{fit}},

\code{p2} is the two-sided p-value for the null hypothesis that \eqn{r_{obs}=r_{fit}} against 
the alternative that \eqn{r_{obs}} is not equal to \eqn{r_{fit}},

where \eqn{r_{obs}} is the rate informed only by direct data, and \eqn{r_{fit}} is the rate
informed by evidence synthesis.  Therefore if the evidence synthesis excludes the 
direct data, then these are interpreted as "conflict" p-values (see Presanis et al. 2013). 

In each case, a small p-value favours the alternative hypothesis.
}
\description{
A test of the hypothesis that the direct data on a disease outcome give the same 
information about that outcome as an indirect evidence synthesis obtained from a fitted \code{\link{disbayes}}
model.   The outcome may be annual incidence, mortality, remission probabilities, 
or prevalence.
}
\details{
Hierarchical models are not supported in this function.
}
\references{
Presanis, A. M., Ohlssen, D., Spiegelhalter, D. J. and De Angelis, D. (2013) 
Conflict diagnostics in directed acyclic graphs, with applications in Bayesian evidence 
synthesis. Statistical Science, 28, 376-397.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/disbayes_hier.R
\name{disbayes_hier}
\alias{disbayes_hier}
\title{Bayesian estimation of chronic disease epidemiology from incomplete data -
hierarchical model for case fatalities.}
\usage{
disbayes_hier(
  data,
  group,
  gender = NULL,
  inc_num = NULL,
  inc_denom = NULL,
  inc_prob = NULL,
  inc_lower = NULL,
  inc_upper = NULL,
  prev_num = NULL,
  prev_denom = NULL,
  prev_prob = NULL,
  prev_lower = NULL,
  prev_upper = NULL,
  mort_num = NULL,
  mort_denom = NULL,
  mort_prob = NULL,
  mort_lower = NULL,
  mort_upper = NULL,
  rem_num = NULL,
  rem_denom = NULL,
  rem_prob = NULL,
  rem_lower = NULL,
  rem_upper = NULL,
  age = "age",
  cf_init = 0.01,
  eqage = 30,
  eqagehi = NULL,
  cf_model = "default",
  inc_model = "smooth",
  rem_model = "const",
  prev_zero = FALSE,
  sprior = c(1, 1, 1),
  hp_fixed = NULL,
  nfold_int_guess = 5,
  nfold_int_upper = 100,
  nfold_slope_guess = 5,
  nfold_slope_upper = 100,
  mean_int_prior = c(0, 10),
  mean_slope_prior = c(5, 5),
  gender_int_priorsd = 0.82,
  gender_slope_priorsd = 0.82,
  inc_prior = c(1.1, 0.1),
  rem_prior = c(1.1, 1),
  method = "opt",
  draws = 1000,
  iter = 10000,
  stan_control = NULL,
  ...
)
}
\arguments{
\item{data}{Data frame containing some of the variables below.  The
variables below are provided as character strings naming columns in this
data frame.   For each disease measure available, one of the following three
combinations of variables must be specified:

(1) numerator and denominator (2) estimate and denominator (3) estimate
with lower and upper credible limits.

Mortality must be supplied, and at least one of incidence and prevalence.
If remission is assumed to be possible, then remission data should also be supplied (see below).

Estimates refer to the probability of having some event within a year, rather than rates.  Rates per year $r$ can be
converted to probabilities $p$ as $p = 1 - exp(-r)$, assuming the rate is constant
within the year.

For estimates based on registry data assumed to cover the whole
population, then the denominator will be the population size.}

\item{group}{Variable in the data representing the area (or other grouping
factor).}

\item{gender}{If \code{NULL} (the default) then the data are one homogenous
gender, and there should be one row per year of age.  Otherwise, set
\code{gender} to a character string naming the variable in the data
representing gender (or other categorical grouping factor).  Gender will then
treated as a fixed additive effect, so the linear effect of gender on log
case fatality is the same in each area.  The data should have one row per
year of age and gender.}

\item{inc_num}{Numerator for the incidence data, assumed to represent the
observed number of new cases within a year among a population of size
\code{inc_denom}.}

\item{inc_denom}{Denominator for the incidence data.

The function \code{\link{ci2num}} can be used to convert a published
estimate and interval for a proportion to an implicit numerator and
denominator.

Note that to include extra uncertainty beyond that implied by a published
interval, the numerator and denominator could be multiplied by a constant,
for example, multiplying both the numerator and denominator by 0.5 would
give the data source half its original weight.}

\item{inc_prob}{Estimate of the incidence probability}

\item{inc_lower}{Lower credible limit for the incidence estimate}

\item{inc_upper}{Upper credible limit for the incidence estimate}

\item{prev_num}{Numerator for the estimate of prevalence, i.e.
number of people currently with a disease.}

\item{prev_denom}{Denominator for the estimate of prevalence (e.g. the size
of the survey used to obtain the prevalence estimate)}

\item{prev_prob}{Estimate of the prevalence probability}

\item{prev_lower}{Lower credible limit for the prevalence estimate}

\item{prev_upper}{Upper credible limit for the prevalence estimate}

\item{mort_num}{Numerator for the estimate of the mortality probability, i.e
number of deaths attributed to the disease under study within a year}

\item{mort_denom}{Denominator for the estimate of the mortality probability (e.g.
the population size, if the estimates were obtained from a comprehensive
register)}

\item{mort_prob}{Estimate of the mortality probability}

\item{mort_lower}{Lower credible limit for the mortality estimate}

\item{mort_upper}{Upper credible limit for the mortality estimate}

\item{rem_num}{Numerator for the estimate of the remission probability, i.e number
of people observed to recover from the disease within a year.

Remission
data should be supplied if remission is permitted in the model, either as
a numerator and denominator or as an estimate and lower credible interval.
Conversely, if no remission data are supplied, then remission is assumed
to be impossible.  These "data" may represent a prior judgement rather than
observation - lower denominators or wider credible limits represent
weaker prior information.}

\item{rem_denom}{Denominator for the estimate of the remission probability}

\item{rem_prob}{Estimate of the remission probability}

\item{rem_lower}{Lower credible limit for the remission estimate}

\item{rem_upper}{Upper credible limit for the remission estimate}

\item{age}{Variable in the data indicating the year of age.  This must
start at age zero, but can end at any age.}

\item{cf_init}{Initial guess at a typical case fatality value, for an
average age.}

\item{eqage}{Case fatalities (and incidence and remission rates) are assumed to be equal for
all ages below this age, inclusive, when using the smoothed model.}

\item{eqagehi}{Case fatalities (and incidence and remission rates) are assumed to be equal for
all ages above this age, inclusive, when using the smoothed model.}

\item{cf_model}{The following alternative models for case fatality are
supported:

\code{"default"} (the default). Random intercepts and slopes, and no
further restriction.

\code{"interceptonly"}.  Random intercepts, but common slopes.

\code{"increasing"}. Case fatality is assumed to be an increasing function
of age (note it is constant below \code{"eqage"} in all models) with a
common slope for all groups.

\code{"common"} Case fatality is an unconstrained function of age
which is common to all areas, i.e. it has the same parameter values in
every area.  This and \code{"increasing_common"} are used in situations
where you want to compare a model with area-specific rates with a single model for
the data aggregated over areas.  Modelling the area-disaggregated data using
a common function for all areas is equivalent to a model for the aggregated data,
and can be statistically compared (using cross-validation) with a model with
area-specific rates.

\code{"increasing_common"} Case fatality is an increasing function of age
which is common to all areas.

\code{"const"} Case fatality is assumed to be constant with age, for all
ages, but different in each area.

\code{"const_common"} Case fatality is a constant over all ages and areas.

In all models, case fatality is a smooth function of age.}

\item{inc_model}{Model for how incidence varies with age.

\code{"smooth"} (the default). Incidence is modelled as a smooth spline
function of age, independently for each area (and gender).

\code{"indep"} Incidence rates for each year of age, area (and gender) are
estimated independently.}

\item{rem_model}{Model for how remission varies with age.  Currently
supported models are \code{"const"} for a constant remission rate over all
ages, \code{"const"} for a smooth spline,  or \code{"indep"} for a different remission rates estimated
independently for each age with no smoothing.}

\item{prev_zero}{If \code{TRUE}, attempt to estimate prevalence at age zero
from the data, as part of the Bayesian model, even if the observed prevalence is zero.
Otherwise (the default) this is assumed to be zero if the count is zero, and estimated
otherwise.}

\item{sprior}{Rates of the exponential prior distributions used to penalise
the coefficients of the spline model.   The default of 1 should adapt
appropriately to the data, but Higher values give stronger smoothing, or
lower values give weaker smoothing,  if required.

This can be a named vector with names \code{"inc","cf","rem"} in any
order, giving the prior smoothness rates for incidence, case fatality and
remission.  If any of these are not smoothed they can be excluded, e.g.
\code{sprior = c(cf=10, inc=1)}.

This can also be an unnamed vector of three elements, where the first
refers to the spline model for incidence, the second for case fatality,
the third for remission. If one of the rates (e.g. remission) is not being
modelled with a spline, any number can be supplied here and it is just
ignored.}

\item{hp_fixed}{A list with one named element for each hyperparameter
to be fixed.  The value should be either
\itemize{
\item a number (to fix the hyperparameter at this number)
\item \code{TRUE} (to fix the hyperparameter at the posterior mode from a training run
where it is not fixed)
}

If the element is either \code{NULL}, \code{FALSE}, or omitted from the list,
then the hyperparameter is given a prior and estimated as part of the Bayesian model.

The hyperparameters that can be fixed are
\itemize{
\item \code{scf} Smoothness parameter for the spline relating case fatality to age.
\item \code{sinc} Smoothness parameter for the spline relating incidence to age.
\item \code{scfmale} Smoothness parameter for the spline defining how the gender
effect relates to age.  Only for models with additive gender and area effects.
\item \code{sd_int} Standard deviation of random intercepts for case fatality.
\item \code{sd_slope} Standard deviation of random slopes for case fatality.
}

For example, to fix the case fatality smoothness to 1.2, fix the incidence
smoothness to its posterior mode, and estimate all the other hyperparameters,
specify \code{hp_fixed = list(scf=1.2, sinc=TRUE)}.}

\item{nfold_int_guess}{Prior guess at the ratio of case fatality between a
high risk (97.5\% quantile) and low risk (2.5\% quantile) area.}

\item{nfold_int_upper}{Prior upper 95\% credible limit for the ratio in
average case fatality between a high risk (97.5\% quantile) and low risk
(2.5\% quantile) area.}

\item{nfold_slope_guess, nfold_slope_upper}{This argument and the next
argument define the prior distribution for the variance in the random
linear effects of age on log case fatality.   They define a prior guess
and upper 95\% credible limit for the ratio of case fatality slopes
between a high trend (97.5\% quantile) and low risk (2.5\% quantile) area.
(Note that the model is not exactly linear, since departures from
linearity are defined through a spline model.  See the Jackson et al.
paper for details.).}

\item{mean_int_prior}{Vector of two elements giving the prior mean and
standard deviation respectively for the mean random intercept for log case
fatality.}

\item{mean_slope_prior}{Vector of two elements giving the prior mean and
standard deviation respectively for the mean random slope for log case
fatality.}

\item{gender_int_priorsd}{Prior standard deviation for the additive effect
of gender on log case fatality}

\item{gender_slope_priorsd}{Prior standard deviation for the additive effect
of gender on the linear age slope of log case fatality}

\item{inc_prior}{Vector of two elements giving the Gamma shape and rate parameters of the
prior for the incidence rate.  Only used if \code{inc_model="indep"}, for each age-specific rate.}

\item{rem_prior}{Vector of two elements giving the Gamma shape and rate parameters of the
prior for the remission rate, used in both \code{rem_model="const"} and \code{rem_model="indep"}.}

\item{method}{String indicating the inference method, defaulting to
\code{"opt"}.

If \code{method="mcmc"} then a sample from the posterior is drawn using Markov Chain Monte Carlo
sampling, via \pkg{rstan}'s \code{\link[rstan:stanmodel-method-sampling]{rstan::sampling()}} function.   This is the most
accurate but the slowest method.

If \code{method="opt"}, then instead of an MCMC sample from the posterior,
\code{disbayes} returns the posterior mode calculated using optimisation, via
\pkg{rstan}'s \code{\link[rstan:stanmodel-method-optimizing]{rstan::optimizing()}} function.
A sample from a normal approximation to the (real-line-transformed)
posterior distribution is drawn in order to obtain credible intervals.

If the optimisation fails to converge (non-zero return code), try increasing the
number of iterations from the default 1000, e.g. \code{disbayes(..., iter=10000, ...)}, or changing the algorithm to \code{disbayes(..., algorithm="Newton", ...)}.

If there is an error message that mentions \code{chol}, then
the computed Hessian matrix is not positive definite at the reported optimum, hence credible intervals
cannot be computed.
This can occur either because of numerical error in computation of the Hessian, or because the
reported optimum is wrong.  If you are willing to believe
the optimum and live without credible intervals, then set \code{draws=0} to skip
computation of the Hessian.   To examine the problematic Hessian, set
\code{hessian=TRUE,draws=0}, then look at the \code{$fit$hessian} component of the
\code{disbayes} return object.   If it can be inverted, do \code{sqrt(diag(solve()))} on the Hessian, and
check for \code{NaN}s, indicating the problematic parameters.
Otherwise, diagonal entries of the Hessian matrix that are very small
may indicate parameters that are poorly identified from the data, leading to computational
problems.

If \code{method="vb"}, then variational Bayes methods are used, via \pkg{rstan}'s
\code{\link[rstan:stanmodel-method-vb]{rstan::vb()}} function.  This is labelled as "experimental" by
\pkg{rstan}.  It might give a better approximation to the posterior
than \code{method="opt"}, but has not been investigated much for \code{disbayes} models.}

\item{draws}{Number of draws from the normal approximation to the posterior
when using \code{method="opt"}.}

\item{iter}{Number of iterations for MCMC sampling, or maximum number of iterations for optimization.}

\item{stan_control}{(\code{method="mcmc"} only). List of options supplied as the \code{control} argument
to \code{\link[rstan:stanmodel-method-sampling]{rstan::sampling()}} in \pkg{rstan} for the main model fit.}

\item{...}{Further arguments passed to \code{\link[rstan:stanmodel-method-sampling]{rstan::sampling()}} to
control MCMC sampling, or \code{\link[rstan:stanmodel-method-optimizing]{rstan::optimizing()}} to control
optimisation, in Stan.}
}
\value{
A list including the following components

\code{call}: Function call that was used.

\code{fit}: An object containing posterior samples from the fitted model,
in the \code{stanfit} format returned by the \code{\link[rstan]{stan}}
function in the \pkg{rstan} package.

\code{method}:  Optimisation method that was chosen.

\code{nage}: Number of years of age in the data

\code{narea}: Number of areas (or other grouping variable that defines the hierarchical model).

\code{ng}: Number of genders (or other categorical variable whose effect is treated as
additive with the area effect).

\code{groups}: Names of the areas (or other grouping variable), taken from the factor levels in the
original data.

\code{genders}: Names of the genders (or other categorical variable), taken from the factor levels in the
original data.

\code{dat}: A list containing the input data in the form of numerators
and denominators.

\code{stan_data}: Full list of data supplied to Stan

\code{stan_inits}: Full list of parameter initial values supplied to Stan

\code{trend}: Whether a time trend was modelled

\code{hp_fixed} Values of any hyperparameters that are fixed during the main model fit.
}
\description{
A variant of \code{\link{disbayes}} in which data from different areas can be
related in a hierarchical model and, optionally, the effect of gender can be
treated as additive with the effect of area.  This is much more computationally
intensive than the basic model in \code{\link{disbayes}}.  Time trends are not
supported in this function.
}
\references{
Jackson C, Zapata-Diomedi B, Woodcock J.
"Bayesian multistate modelling of incomplete chronic disease burden data"
\url{https://arxiv.org/abs/2111.14100}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/disbayes-package.R
\docType{data}
\name{ihdengland}
\alias{ihdengland}
\title{Ischemic heart disease in England}
\format{
A data frame with columns:

\code{sex}: \code{"male"} or \code{"female"}. 

\code{ageyr}. Year of age. 

\code{location}. Name of the location, which is either a city region or region in England. 

\code{num_mort}. Numerator behind the estimate of mortality 

\code{num_inc}. Numerator behind the estimate of incidence

\code{num_prev}. Numerator behind the estimate of prevalence

\code{denom_mort}. Denominator behind the estimate of mortality 

\code{denom_inc}.  Denominator behind the estimate of incidence

\code{denom_prev}. Denominator behind the estimate of prevalence
}
\source{
Global Burden of Disease, 2017
}
\usage{
ihdengland
}
\description{
Ischemic heart disease in England
}
\details{
The data were processed to

* change the geography to refer to England city regions and the remaining English regions,

* change counts by 5-year age groups to estimated 1-year counts,

* obtain estimated numerators and denominators from the published point estimates and uncertainty intervals.
A point estimate of the risk is equivalent to the numerator divided by the denominator. The denominator is
related to the extent of uncertainty around this estimate, and obtained using the Bayesian method
implemented in \code{\link{ci2num}}. 

The script given in \url{https://github.com/chjackson/disbayes/blob/master/data-raw/gbd_process.Rmd} shows
these steps.
}
\references{
Jackson C, Zapata-Diomedi B, Woodcock J. "Bayesian multistate modelling of incomplete chronic disease burden data" \url{https://arxiv.org/abs/2111.14100}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loo.R
\name{loo_indiv}
\alias{loo_indiv}
\alias{looi_disbayes}
\title{Extract observation-specific contributions from a disbayes leave-one-out cross validation}
\usage{
loo_indiv(x, agg = FALSE)

looi_disbayes(x, agg = FALSE)
}
\arguments{
\item{x}{For \code{loo_indiv}, an object returned by \code{\link{loo.disbayes}}.   For \code{looi_disbayes}, an object returned by \code{\link{disbayes}}.}

\item{agg}{If \code{TRUE} then the observation-specific contributions are aggregated over
outcome type, returning a data frame with one row for each of incidence, prevalence, mortality
and remission (if remission is included in the model), and one column for each of \code{"elpd_loo"},
\code{"p_loo"} and \code{"looic"}.}
}
\value{
A data frame with one row per observed age-specific mortality, incidence, prevalence and/or
remission age-specific data-point, containing leave-one-out cross validation statistics representing how
well the model would predict that observation if it were left out of the fit. 

These are computed with the \pkg{loo} package.

\code{loo_indiv} acts on the objects that are returned by running \code{\link{loo}} on \code{\link{disbayes}}
objects.  \code{\link{looi_disbayes}} acts directly on  \code{\link{disbayes}}
objects.  Both of those functions return a data frame with LOO contributions for each data point.
}
\description{
Extract observation-specific contributions from a disbayes leave-one-out cross validation
}
\section{Functions}{
\itemize{
\item \code{looi_disbayes()}: Observation-level leave-one-out cross validation statistics for a disbayes model

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy.R
\name{tidy.disbayes}
\alias{tidy.disbayes}
\alias{tidy.disbayes_hier}
\title{Form a tidy data frame from the estimates from a disbayes fit}
\usage{
\method{tidy}{disbayes}(x, startyear = 1, ...)

\method{tidy}{disbayes_hier}(x, ...)
}
\arguments{
\item{x}{Object returned by \code{\link{disbayes}}}

\item{startyear}{Only used for models with time trends.  Numeric year represented by year 1 in the data. For example, set this to 1918 to convert years 1-100 to years 1918-2017.}

\item{...}{Other arguments (currently unused)}
}
\value{
A data frame with one row per model parameter, giving summary statistics
for the posterior distribution for that parameter.   For array parameters, e.g. those
that depend on age or area, then the age and area are returned in separate columns,
to make it easier to summarise and plot the results, e.g. using \pkg{ggplot2}.

Model parameters might include, depending on the model specification,
\itemize{
\item \code{cf}, \code{inc}, \code{rem}: Case fatality, incidence, remission rates
\item \code{inc_prob}, \code{rem_prob}, \code{mort_prob}, \code{cf_prob}:  Annual incidence, remission, mortality and case fatality risks (probabilities).
\item \code{prev_prob} Prevalence (probability).
\item \code{state_probs} State occupancy probabilities.
\item \code{beta}, \code{beta_inc} Coefficients of the spline basis for case fatality and incidence respectively.
\item \code{lambda_cf}, \code{lambda_inc} Smoothness parameters of the spline functions.
\item \code{prevzero} Prevalence at age zero
\item \code{cfbase} Case fatality at the baseline age (only in models where case fatality is increasing).
\item \code{dcf} Annual increments in case fatality (only in models where case fatality is increasing).
\item \code{bias_loghr} Log hazard ratio describing bias in case fatality between datasets (only in models where \code{bias_model} has been set).
}

For models with time trends:
\itemize{
\item \code{cf_yr}, \code{inc_yr}, \code{state_probs_yr} Case fatality rates, incidence rates and state occupancy probabilities in years prior to the current year.  \code{cf} and \code{inc} refer to the rates for the current year, the one represented in the data.
}

Only for hierarchical models:
\itemize{
\item \code{mean_inter}, \code{mean_slope},\code{sd_inter},\code{sd_slope}.  Mean and standard deviation for random effects distribution for the intercept and slope of log case fatality.
\item \code{lambda_cf_male}, \code{lambda_inc_male}.  Smoothness of the additive gender effect on case fatality and incidence.
\item \code{bareat} Area-level contribution to spline basis coefficients.
\item \code{barea}  Normalised spline basis coefficients.
}
}
\description{
Simply call this after fitting disbayes, as, e.g.

\if{html}{\out{<div class="sourceCode">}}\preformatted{res <- disbayes(...)
tidy(res)
}\if{html}{\out{</div>}}
}
\section{Functions}{
\itemize{
\item \code{tidy(disbayes_hier)}: Tidy method for hierarchical disbayes models

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/disbayes-package.R
\docType{data}
\name{ihdtrends}
\alias{ihdtrends}
\title{Trends in ischemic heart disease in England}
\format{
A data frame with columns:

\code{gender}: \code{"male"} or \code{"female"}. 

\code{age}: Year of age. 

\code{year}: Calendar year.

\code{p2017}: Estimated ratio between the outcome in the calendar
year and the outcome in 2017.

\code{outcome}: Outcome referred to (incidence or case fatality).
}
\source{
Scarborough, P., Wickramasinghe, K., Bhatnagar, P. and Rayner, M. (2011) Trends in coronary heart disease, 1961-2001. British Heart Foundation.

Smolina, K., Wright, F. L., Rayner, M. and Goldacre, M. J. (2012) Determinants of the decline in mortality from acute myocardial infarction in England between 2002 and 2010: linked national database study. BMJ, 344.

British Heart Foundation (2020) Heart and Circulatory Disease Statistics 2020. British Heart Foundation.
}
\usage{
ihdtrends
}
\description{
Trends in ischemic heart disease in England
}
\details{
The data were interpolated and smoothed to produce a matrix by year of age and
calendar year, using the script at \url{https://github.com/chjackson/disbayes/blob/master/data-raw/trends.r}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loo.R
\name{loo.disbayes}
\alias{loo.disbayes}
\title{Leave-one-out cross validation for disbayes models}
\usage{
\method{loo}{disbayes}(x, outcome = "overall", ...)
}
\arguments{
\item{x}{A model fitted by \code{\link{disbayes}}.  Any of the computation methods
are supported.}

\item{outcome}{Either \code{"overall"}, to assess the fit to all data, or 
one of \code{"inc"}, \code{"prev"}, \code{"mort"} or \code{"rem"}, to assess the fit
to the incidence data, prevalence data, mortalidy data or remission data, respectively.}

\item{...}{Other arguments (currently unused).}
}
\value{
An object of class \code{"loo"} as defined by the \pkg{loo} package.
}
\description{
Leave-one-out cross validation for disbayes models
}
\seealso{
\code{\link{loo_indiv}} to return tidied observation-specific contributions
to the overall model fit computed here.
}
